from __future__ import print_function, division, absolute_import

import mock

from . import fixture
from . import stubs

from rhsm import certificate2
from subscription_manager import entbranding
from subscription_manager import rhelentbranding
from subscription_manager import injection as inj


# The certificate2.Product is basically just a data structure
#  so we can use the real one. No methods that need mocking.
class StubProduct(certificate2.Product):
    pass


class DefaultStubProduct(StubProduct):
    def __init__(self, id=123, name="Awesome OS",
                 brand_type='OS', brand_name='Awesome OS super'):

        super(DefaultStubProduct, self).__init__(id=id, name=name, brand_type=brand_type,
                                                 brand_name=brand_name)


# The installed product ids don't have brand_type/brand_name, just the
# Product from the ent cert
class DefaultStubInstalledProduct(StubProduct):
    def __init__(self, id=123, name="Awesome OS",
                 brand_type=None, brand_name=None):

        super(DefaultStubInstalledProduct, self).__init__(id=id, name=name, brand_type=brand_type,
                                                 brand_name=brand_name)


class BaseBrandFixture(fixture.SubManFixture):
    current_brand = "Current Awesome OS Brand"

    def setUp(self):
        super(BaseBrandFixture, self).setUp()
        self.brand_file_write_patcher = mock.patch("subscription_manager.entbranding.BrandFile.write",
                                              name="MockBrandFile.write")
        self.mock_write = self.brand_file_write_patcher.start()

        self.brand_file_read_patcher = mock.patch("subscription_manager.entbranding.BrandFile.read",
                                             return_value="%s\n" % self.current_brand,
                                             name="MockBrandFile.read")
        self.mock_brand_read = self.brand_file_read_patcher.start()

        self.mock_install_patcher = mock.patch("subscription_manager.rhelentbranding.RHELBrandInstaller._install",
                                               name="Mock_install_branding")

        self.mock_install = self.mock_install_patcher.start()

    def tearDown(self):
        super(BaseBrandFixture, self).tearDown()
        self.brand_file_write_patcher.stop()
        self.brand_file_read_patcher.stop()
        self.mock_install_patcher.stop()


# never write out the file to disk
class TestBrandInstaller(BaseBrandFixture):
    brand_installer_class = entbranding.BrandInstaller

    def test_init_empty_cert_list(self):
        self.brand_installer_class([])

    def test_init_no_ent_cert(self):
        self.brand_installer_class()


class TestRHELBrandInstaller(BaseBrandFixture):
    brand_installer_class = rhelentbranding.RHELBrandInstaller

    def test_init_empty_cert_list(self):
        self.brand_installer_class([])

    def test_init_no_ent_cert(self):
        self.brand_installer_class()

    def test_no_installed_products_no_ent_certs(self):
        brand_installer = self.brand_installer_class([])
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_no_ent_certs_installed_products(self):
        prod_dir = stubs.StubProductDirectory(pids=[1, 2, 3, 4, 5])
        inj.provide(inj.PROD_DIR, prod_dir)
        brand_installer = self.brand_installer_class([])
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def _inj_mock_ent_dir(self, ents=None):
        ent_list = ents or []
        mock_ent_dir = mock.NonCallableMock(name='MockEntDir')
        mock_ent_dir.list_valid.return_value = ent_list
        inj.provide(inj.ENT_DIR, mock_ent_dir)
        return mock_ent_dir

    def test(self):
        stub_product = DefaultStubProduct()
        stub_installed_product = DefaultStubInstalledProduct()

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]

        self._inj_mock_ent_dir([mock_ent_cert])
        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertTrue(self.mock_install.called)
        call_args = self.mock_install.call_args
        brand_arg = call_args[0][0]
        self.assertTrue(isinstance(brand_arg, entbranding.ProductBrand))
        self.assertTrue(isinstance(brand_arg, rhelentbranding.RHELProductBrand))
        self.assertEqual('Awesome OS super', brand_arg.name)

    def test_no_need_to_update_branding(self):
        stub_product = StubProduct(id=123, brand_type='OS',
                                   name="Some name",
                                   brand_name=self.current_brand)

        stub_installed_product = StubProduct(id=123, name="Some name")

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]

        self._inj_mock_ent_dir([mock_ent_cert])
        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_no_brand_type_on_product(self):
        # no brand_type
        stub_product = StubProduct(id=123, name="Awesome OS Super")
        stub_installed_product = StubProduct(id=123, name="Awesome OS Super")

        # simulate a old style Product, shouldn't happen
        del stub_product.brand_type
        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]

        self._inj_mock_ent_dir([mock_ent_cert])
        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_none_product_name_none_brand_name_on_product(self):
        # can't create a certificate2.Product without a name
        # so create it with and undo it. This should never happen
        stub_product = StubProduct(name="placeholder", id=123, brand_type='OS')
        stub_product.name = None
        stub_product.brand_name = None

        stub_installed_product = StubProduct(name="placeholder", id=123)

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_none_brand_name_on_product(self):
        stub_product = StubProduct(id=123, name="An Awesome OS", brand_type='OS')
        stub_product.name = None

        stub_installed_product = StubProduct(id=123, name="An Awesome OS")

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_none_brand_type_none_brand_name_on_product(self):
        stub_product = StubProduct(id=123, name="An Awesome OS")
        stub_product.name = None

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_none_brand_type_brand_name_on_product(self):
        stub_product = StubProduct(id=123, name="An Awesome OS",
                                   brand_name="Branded Awesome OS")
        stub_product.name = None

        stub_installed_product = StubProduct(id=123, name="An Awesome OS")
        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_wrong_brand_type_brand_name_on_product(self):
        stub_product = StubProduct(id=123, name="An Awesome OS",
                                   brand_type='Middleware',
                                   brand_name="Branded Awesome OS")

        stub_installed_product = StubProduct(id=123, name="An Awesome OS")

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_wrong_brand_type_none_brand_name_on_product(self):
        stub_product = StubProduct(id=123, name="An Awesome OS",
                                   brand_type='Middleware')

        stub_installed_product = StubProduct(id=123, name="An Awesome OS")

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        self._inj_mock_ent_dir([mock_ent_cert])

        brand_installer = self.brand_installer_class()
        brand_installer.install()

        self.assertFalse(self.mock_install.called)

    def test_multiple_matching_branded_products(self):
        stub_product = DefaultStubProduct()
        stub_installed_product = DefaultStubInstalledProduct()

        stub_product_2 = StubProduct(id=321, brand_type='OS', name="Awesome",
                                     brand_name="Slightly Different Awesome OS Super")

        stub_installed_product_2 = StubProduct(id=321, name="Awesome")

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id,
                                                             stub_installed_product_2.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product, stub_product_2]

        brand_installer = self.brand_installer_class([mock_ent_cert])
        brand_installer.install()

        # we give up in this case and leave things as is
        self.assertFalse(self.mock_install.called)

    def test_multiple_branded_ent_certs_for_installed_product(self):
        stub_product = DefaultStubProduct()
        stub_installed_product = DefaultStubInstalledProduct()

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_installed_product.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]

        mock_ent_cert_2 = mock.Mock(name='MockEntCert2')
        mock_ent_cert_2.products = [stub_product]

        brand_installer = self.brand_installer_class([mock_ent_cert, mock_ent_cert_2])
        brand_installer.install()

        self.assertTrue(self.mock_install.called)


class TestBrandsInstaller(TestBrandInstaller):
    brand_installer_class = entbranding.BrandsInstaller


class StubEmptyBrandsInstaller(entbranding.BrandsInstaller):
    def _get_brand_installers(self):
        return []


class TestEmptyBrandsInstaller(TestBrandInstaller):
    brand_installer_class = StubEmptyBrandsInstaller


class StubMultipleBrandsInstaller(entbranding.BrandsInstaller):
    def _get_brand_installers(self):
        mock_brand_installer_1 = mock.Mock()
        mock_brand_installer_2 = mock.Mock()
        return [mock_brand_installer_1, mock_brand_installer_2]


class TestMultipleBrandsInstaller(TestBrandInstaller):
    brand_installer_class = StubMultipleBrandsInstaller


class StubRhelAndMockBrandsInstaller(rhelentbranding.RHELBrandsInstaller):
    def _get_brand_installers(self):
        return [mock.Mock(),
                rhelentbranding.RHELBrandInstaller(self.ent_certs),
                mock.Mock()]


class TestRhelAndMockBrandsInstaller(TestRHELBrandInstaller):
    """Test a BrandsInstaller with a RHELBrandInstaller and a mock"""
    brand_installer_class = StubRhelAndMockBrandsInstaller

    def test(self):
        stub_product = DefaultStubProduct()

        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_product.id]

        inj.provide(inj.PROD_DIR, mock_prod_dir)

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]

        brand_installer = self.brand_installer_class([mock_ent_cert])
        brand_installer.install()

        self.assertTrue(self.mock_install.called)
        call_args = self.mock_install.call_args
        brand_arg = call_args[0][0]
        self.assertTrue(isinstance(brand_arg, entbranding.ProductBrand))
        self.assertTrue(isinstance(brand_arg, rhelentbranding.RHELProductBrand))
        self.assertEqual("Awesome OS super", brand_arg.name)

        # verify the install on all the installers got called
        count = 0
        for bi in brand_installer.brand_installers:
            if isinstance(bi, mock.Mock):
                self.assertTrue(bi.install.called)
                count = count + 1
        self.assertEqual(2, count)


class TestBrand(fixture.SubManFixture):
    def test_no_no(self):
        current_brand = entbranding.Brand()
        product_brand = entbranding.Brand()
        # current brand and product none/unset
        self.assertTrue(current_brand.is_outdated_by(product_brand))

    def test_product_brand_none(self):
        # new name is None, so newer consumer > current
        current_brand = entbranding.Brand()
        current_brand.name = "Awesome OS"
        product_brand = entbranding.Brand()
        product_brand.name = None
        self.assertFalse(current_brand.is_outdated_by(product_brand))

    def test_no_current_name(self):
        # if old name doesnt exist, new name is newer
        current_brand = entbranding.Brand()
        product_brand = entbranding.Brand()
        product_brand.name = "Awesome OS"
        self.assertTrue(current_brand.is_outdated_by(product_brand))

    def test_same(self):
        # name is the same, so not new
        current_brand = entbranding.Brand()
        current_brand.name = "Awesome OS"
        product_brand = entbranding.Brand()
        product_brand.name = "Awesome OS"
        self.assertFalse(current_brand.is_outdated_by(product_brand))

    def test_product_name_ne_current_name(self):
        # a new branded name
        current_brand = entbranding.Brand()
        current_brand.name = "Awesome OS"
        product_brand = entbranding.Brand()
        product_brand.name = "New Awesome OS"

        self.assertTrue(current_brand.is_outdated_by(product_brand))


class TestBrandPicker(BaseBrandFixture):
    def test_init(self):
        entbranding.BrandPicker([])


class TestRHELBrandPicker(BaseBrandFixture):
    @mock.patch("subscription_manager.rhelentbranding.RHELBrandPicker._get_branded_cert_products",
                return_value=[(mock.Mock(name="Mock Certificate 1"), DefaultStubProduct()),
                              (mock.Mock(name="Mock Certificate 2"), DefaultStubProduct())])
    def test_more_than_one_ent_cert_with_branding(self, mock_branded_certs):
        brand_picker = rhelentbranding.RHELBrandPicker([])
        brand = brand_picker.get_brand()
        self.assertEqual("Awesome OS super", brand.name)

    @mock.patch("subscription_manager.rhelentbranding.RHELBrandPicker._get_branded_cert_products",
                return_value=[])
    def test_branded_certs_returns_empty(self, mock_branded_certs):

        brand_picker = rhelentbranding.RHELBrandPicker([])
        brand = brand_picker.get_brand()

        self.assertEqual(None, brand)

    def test_get_brand(self):
        # inj a prod dir with some installed products
        stub_installed_product = DefaultStubInstalledProduct()
        stub_product = DefaultStubProduct()

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [stub_installed_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        mock_ent_cert = mock.Mock()
        mock_ent_cert.products = [stub_product]
        ent_certs = [mock_ent_cert]

        brand_picker = rhelentbranding.RHELBrandPicker(ent_certs)
        brand = brand_picker.get_brand()
        self.assertTrue("Mock Product", brand.name)

    def test_get_brand_multiple_ents_with_branding_same_name(self):
        # inj a prod dir with some installed products
        stub_product = DefaultStubProduct()

        stub_product_2 = DefaultStubProduct()

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [stub_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        mock_ent_cert = mock.Mock()
        mock_ent_cert.products = [stub_product]

        mock_ent_cert_2 = mock.Mock()
        mock_ent_cert_2.products = [stub_product_2]

        ent_certs = [mock_ent_cert, mock_ent_cert_2]

        brand_picker = rhelentbranding.RHELBrandPicker(ent_certs)
        brand = brand_picker.get_brand()
        self.assertTrue(brand is not None)
        self.assertTrue("Awesome OS", brand.name)

    def test_get_brand_multiple_ents_with_branding_different_branded_name(self):
        # inj a prod dir with some installed products
        stub_product = DefaultStubProduct()

        # same product id, different name
        stub_product_2 = StubProduct(id=123, brand_type='OS',
                                     name='A Different Stub Product',
                                     brand_name='A Different branded Stub Product')

        mock_product_dir = mock.NonCallableMock()
        # note stub_product.id=123 will match the Product from both ents
        mock_product_dir.get_installed_products.return_value = [stub_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        mock_ent_cert = mock.Mock()
        mock_ent_cert.products = [stub_product]

        mock_ent_cert_2 = mock.Mock()
        mock_ent_cert_2.products = [stub_product_2]
        ent_certs = [mock_ent_cert, mock_ent_cert_2]

        brand_picker = rhelentbranding.RHELBrandPicker(ent_certs)
        brand = brand_picker.get_brand()
        self.assertTrue(brand is None)

    def test_get_brand_not_installed(self):

        stub_product = DefaultStubProduct()

        # note, no 'brand_type' set
        other_stub_product = StubProduct(id=321, name="A Non Branded Product")

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [other_stub_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        mock_ent_cert = mock.Mock()
        mock_ent_cert.products = [stub_product]
        ent_certs = [mock_ent_cert]

        brand_picker = rhelentbranding.RHELBrandPicker(ent_certs)
        brand = brand_picker.get_brand()
        self.assertTrue(brand is None)

    def test_get_brand_branded_unknown_brand_type(self):

        stub_installed_product = StubProduct(id=123, name="Stub Product Name")

        # note, no 'brand_type' attribute
        other_stub_installed_product = StubProduct(id=321, name='A Non Branded Product')

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [stub_installed_product.id,
                                                                other_stub_installed_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        stub_product = StubProduct(id=123, brand_type="middleware",
                                   name="Stub Product Name", brand_name="Awesome Middleware")
        mock_ent_cert = mock.Mock()
        mock_ent_cert.products = [stub_product]
        ent_certs = [mock_ent_cert]

        # NOTE: this looks like a branded product, except the brand type is one
        # the RHELBrandPicker doesn't know
        brand_picker = rhelentbranding.RHELBrandPicker(ent_certs)
        brand = brand_picker.get_brand()
        self.assertTrue(brand is None)

    def test_is_installed_rhel_branded_product_not_installed(self):
        brand_picker = rhelentbranding.RHELBrandPicker([])
        stub_product = DefaultStubProduct()
        # note no installed products in injected installed products
        irp = brand_picker._is_installed_rhel_branded_product(stub_product)
        self.assertFalse(irp)

    def test_is_installed_rhel_branded_product_is_installed(self):
        stub_product = DefaultStubProduct()

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [stub_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        brand_picker = rhelentbranding.RHELBrandPicker([])
        irp = brand_picker._is_installed_rhel_branded_product(stub_product)
        self.assertTrue(irp)

    def test_get_installed_branded_products(self):
        stub_product = DefaultStubProduct()

        mock_product_dir = mock.NonCallableMock()
        mock_product_dir.get_installed_products.return_value = [stub_product.id]
        inj.provide(inj.PROD_DIR, mock_product_dir)

        brand_picker = rhelentbranding.RHELBrandPicker([])
        irp = brand_picker._is_installed_rhel_branded_product(stub_product)
        self.assertTrue(irp)


class TestRHELBrandPickerIsRHELBrandedProduct(BaseBrandFixture):
    def setUp(self):
        super(TestRHELBrandPickerIsRHELBrandedProduct, self).setUp()
        self.brand_picker = rhelentbranding.RHELBrandPicker([])

    def _check_branded(self, product):
        return self.brand_picker._is_rhel_branded_product(product)

    def test(self):
        stub_product = DefaultStubProduct()
        self.assertTrue(self._check_branded(stub_product))

    def test_none_brand_type(self):
        # brand_type is set to None (not specified)
        none_brand_type_stub_product = StubProduct(name="Awesome NoOS", id=123)
        self.assertFalse(self._check_branded(none_brand_type_stub_product))

    def test_no_brand_type(self):
        # a Product with no brand_type attribute, like old versions
        # are not branded
        no_brand_type_stub_product = DefaultStubProduct()
        del no_brand_type_stub_product.brand_type
        self.assertFalse(self._check_branded(no_brand_type_stub_product))

    def test_empty_brand_type(self):
        # At the moment, "" != "OS", so brand_type="" is not a rhel
        empty_brand_type_stub_product = DefaultStubProduct(brand_type="")
        self.assertFalse(self._check_branded(empty_brand_type_stub_product))

    def test_none_brand_name(self):
        # product.brand_name is none
        none_brand_name_stub_product = DefaultStubProduct()
        none_brand_name_stub_product.brand_name = None
        self.assertFalse(self._check_branded(none_brand_name_stub_product))

    def test_no_brand_name(self):
        # no Product.brand_name at all, ala old Product objects
        no_brand_name_stub_product = DefaultStubProduct()
        del no_brand_name_stub_product.brand_name
        self.assertFalse(self._check_branded(no_brand_name_stub_product))

    def test_empty_brand_name(self):
        # product.brand_name is ""
        empty_brand_name_stub_product = DefaultStubProduct(brand_name="")
        self.assertFalse(self._check_branded(empty_brand_name_stub_product))


class TestProductBrand(BaseBrandFixture):
    brand_class = entbranding.ProductBrand

    def test_init(self):
        self.brand_class("Awesome OS")

    def test_brand(self):
        brand = self.brand_class("Awesome OS")
        self.assertEqual("Awesome OS", brand.name)

    def test_brand_save(self):
        brand = self.brand_class("Foo")
        brand.save()
        self.mock_write.assert_called_with("Foo\n")

    def test_exception_on_save(self):
        self.mock_write.side_effect = IOError
        brand = self.brand_class("Foo")
        self.assertRaises(IOError, brand.save)

    def test_from_product(self):
        stub_product = DefaultStubProduct()
        brand = self.brand_class.from_product(stub_product)
        self.assertEqual(stub_product.brand_name, brand.name)

    def test_from_product_brand_name_none(self):
        stub_product = DefaultStubProduct(brand_name=None)
        brand = self.brand_class.from_product(stub_product)
        self.assertEqual(stub_product.brand_name, brand.name)
        self.assertTrue(brand.name is None)

    # ProductBrand shouldn't care about the Product.brand_type,
    # that's BrandPicker's job
    def test_from_product_brand_type_none(self):
        stub_product = DefaultStubProduct(brand_type=None)
        brand = self.brand_class.from_product(stub_product)
        self.assertEqual(stub_product.brand_name, brand.name)

    # An empty string is just a name, a poor name
    # ie, no casting to None, etc.
    # NOTE: we basically ignore brand_name="" in RHELBrandPicker
    # so we can't install a branded name of "", which is okay
    # since brandbot doesn't know what to do with that.
    def test_from_product_brand_name_empty_string(self):
        stub_product = DefaultStubProduct(brand_name="")
        brand = self.brand_class.from_product(stub_product)
        self.assertEqual(stub_product.brand_name, brand.name)
        self.assertEqual("", brand.name)

    # see comment about test_from_product_brand_type_none
    def test_from_product_brand_type_empty_string(self):
        stub_product = DefaultStubProduct(brand_type="")
        brand = self.brand_class.from_product(stub_product)
        self.assertEqual(stub_product.brand_name, brand.name)

    def test_format_brand(self):
        fb = self.brand_class.format_brand('Blip')
        self.assert_string_equals(fb, 'Blip\n')

        fb = self.brand_class.format_brand('')
        self.assert_string_equals(fb, '\n')

        fb = self.brand_class.format_brand('Foo\n')
        self.assert_string_equals(fb, 'Foo\n')


class TestRHELProductBrand(TestProductBrand):
    brand_class = rhelentbranding.RHELProductBrand


class TestCurrentBrand(BaseBrandFixture):
    brand_class = entbranding.CurrentBrand

    def test_init(self):
        self.brand_class()

    def test_load(self):
        cb = self.brand_class()
        self.assertEqual(self.current_brand, cb.name)

    def test_unformat_brand(self):
        self.assertEqual("foo", self.brand_class.unformat_brand("foo"))
        self.assertEqual("foo", self.brand_class.unformat_brand("foo\n"))

    def test_unformat_brand_none(self):
        self.assertTrue(self.brand_class.unformat_brand(None) is None)

    def test_io_exception_on_brandfile_read(self):
        self.mock_brand_read.side_effect = IOError
        cb = self.brand_class()
        self.assertTrue(cb.name is None)

    def test_exception_on_brandfile_read(self):
        self.mock_brand_read.side_effect = Exception
        self.assertRaises(Exception, self.brand_class)


class TestRHELCurrentBrand(TestCurrentBrand):
    brand_class = rhelentbranding.RHELCurrentBrand


class TestBrandFile(fixture.SubManFixture):
    brandfile_class = entbranding.BrandFile

    def test_init(self):
        self.brandfile_class()
        self.assertEqual("/var/lib/rhsm/branded_name", self.brandfile_class.path)

    def test_write(self):
        brand_file = self.brandfile_class()
        mo = mock.mock_open()
        with mock.patch('subscription_manager.entbranding.open', mo, create=True):
            brand_file.write("Foo OS")
        mo.assert_called_once_with('/var/lib/rhsm/branded_name', 'w')

    def test_read(self):
        brand_file = self.brandfile_class()
        brand_string = "Some Branding Info"
        mo = mock.mock_open(read_data=brand_string)
        with mock.patch('subscription_manager.entbranding.open', mo, create=True):
            b = brand_file.read()
        self.assert_string_equals(brand_string, b)


class TestRHELBrandFile(TestBrandFile):
    brandfile_class = rhelentbranding.RHELBrandFile



from __future__ import print_function, division, absolute_import

import unittest

from rhsm.certificate import OID


class OIDTests(unittest.TestCase):

    def setUp(self):
        self.oid = OID("1.2.3.4.5.6.7")

    def test_length(self):
        self.assertEqual(7, len(self.oid))

    def test_match_positive(self):
        self.assertTrue(self.oid.match(OID("1.2.3.4.5.6.7")))
        self.assertTrue(self.oid.match(OID("1.")))
        self.assertTrue(self.oid.match(OID("1.2.3.")))
        self.assertTrue(self.oid.match(OID("1.*.3.4.5.6.7")))
        self.assertTrue(self.oid.match(OID("1.*.3.4.*.6.*")))
        self.assertTrue(self.oid.match(OID(".*")))
        self.assertTrue(self.oid.match(OID(".5.6.7")))
        self.assertTrue(self.oid.match(OID(".7")))

    def test_match_negative(self):
        self.assertFalse(self.oid.match(OID("1.2.3.4.5.9.7")))
        self.assertFalse(self.oid.match(OID("1.2.4.")))
        self.assertFalse(self.oid.match(OID("1.*.4.")))
        # * matches only one item
        self.assertFalse(self.oid.match(OID("1.*")))

        # Not an OID
        self.assertFalse(self.oid.match("1.2.3.4.5.6.7"))



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import json
import shutil
import tempfile
import unittest
from .fixture import set_up_mock_sp_store

import os
import mock


class SyspurposeStoreInterfaceTests(unittest.TestCase):

    def setUp(self):
        temp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, temp_dir)
        mock_syspurpose_file = os.path.join(temp_dir, "mock_syspurpose.json")
        syspurpose_values = {}
        with open(mock_syspurpose_file, 'w') as f:
            json.dump(syspurpose_values, f)
            f.flush()

        from subscription_manager import syspurposelib

        self.syspurposelib = syspurposelib
        syspurposelib.USER_SYSPURPOSE = mock_syspurpose_file

        syspurpose_patch = mock.patch('subscription_manager.syspurposelib.SyncedStore')
        self.mock_sp_store = syspurpose_patch.start()
        self.mock_sp_store, self.mock_sp_store_contents = set_up_mock_sp_store(self.mock_sp_store)
        self.addCleanup(syspurpose_patch.stop)

    def _set_up_mock_sp_store(self):
        """
        Sets up the mock syspurpose store with methods that are mock versions of the real deal.
        Allows us to test in the absence of the syspurpose module.
        :return:
        """
        contents = {}
        self.mock_sp_store_contents = contents

        def set(item, value):
            contents[item] = value

        def read(path, raise_on_error=False):
            return self.mock_sp_store

        def unset(item):
            contents[item] = None

        def add(item, value):
            current = contents.get(item, [])
            if value not in current:
                current.append(value)
            contents[item] = current

        def remove(item, value):
            current = contents.get(item)
            if current is not None and isinstance(current, list) and value in current:
                current.remove(value)

        self.mock_sp_store.set = mock.Mock(side_effect=set)
        self.mock_sp_store.read = mock.Mock(side_effect=read)
        self.mock_sp_store.unset = mock.Mock(side_effect=unset)
        self.mock_sp_store.add = mock.Mock(side_effect=add)
        self.mock_sp_store.remove = mock.Mock(side_effect=remove)
        self.mock_sp_store.contents = self.mock_sp_store_contents

    def tearDown(self):
        self.syspurposelib.USER_SYSPURPOSE = "/etc/rhsm/syspurpose/syspurpose.json"

    def test_save_sla_to_syspurpose_metadata_sla_is_set_when_syspurpose_module_exists(self):
        """
        Tests that the syspurpose sla is set through the syspurposestore interface
        when the syspurpose module is available for import.
        """
        self.syspurposelib.save_sla_to_syspurpose_metadata("Freemium")

        contents = self.syspurposelib.read_syspurpose()
        self.assertEqual(contents.get("service_level_agreement"), "Freemium")

    def test_save_sla_to_syspurpose_metadata_sla_is_not_set_when_None_is_provided(self):
        """
        Tests that the syspurpose sla is not set through the syspurposestore interface
        when None is passed to save_sla_to_syspurpose_metadata method.
        """
        self.syspurposelib.save_sla_to_syspurpose_metadata(None)

        contents = self.syspurposelib.read_syspurpose()
        self.assertEqual(contents.get("service_level_agreement"), None)

    def test_save_sla_to_syspurpose_metadata_sla_is_not_set_when_empty_string_is_provided(self):
        """
        Tests that the syspurpose sla is not set through the syspurposestore interface
        when an empty string is passed to save_sla_to_syspurpose_metadata method.
        """
        self.syspurposelib.save_sla_to_syspurpose_metadata("")

        contents = self.syspurposelib.read_syspurpose()
        self.assertEqual(contents.get("service_level_agreement"), None)

    def test_save_sla_to_syspurpose_metadata_sla_is_not_set_when_syspurpose_module_does_not_exist(self):
        """
        Tests that the syspurpose sla is NOT set through the syspurposestore interface
        when the syspurpose module is not available for import.
        """
        # Remove SyspurposeStore and USER_SYSPURPOSE from syspurposestore_interface's scope temporarily
        # to simulate that importing them failed.
        tmp_syspurpose_store = self.syspurposelib.SyncedStore
        tmp_user_syspurpose = self.syspurposelib.USER_SYSPURPOSE
        del self.syspurposelib.SyncedStore
        del self.syspurposelib.USER_SYSPURPOSE

        self.syspurposelib.save_sla_to_syspurpose_metadata("Freemium")

        # Add SyspurposeStore and USER_SYSPURPOSE back to syspurposestore_interface's scope
        self.syspurposelib.SyncedStore = tmp_syspurpose_store
        self.syspurposelib.USER_SYSPURPOSE = tmp_user_syspurpose

        # Check that the contents of the syspurpose.json are empty (thus sla was not set)
        contents = self.syspurposelib.read_syspurpose()
        self.assertFalse(contents)



# -*- coding: utf-8 -*-#

from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010, 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import re
import six
from . import fixture

from iniparse import RawConfigParser, SafeConfigParser
from mock import Mock, patch, MagicMock
import tempfile
import os
from iniparse import ConfigParser

from .stubs import StubProductCertificate, \
        StubProduct, StubEntitlementCertificate, StubContent, \
        StubProductDirectory, StubConsumerIdentity, StubEntitlementDirectory
from subscription_manager.repolib import RepoActionInvoker, \
        RepoUpdateActionCommand, YumReleaseverSource, YumPluginManager
from subscription_manager.repofile import Repo, TidyWriter, YumRepoFile
from subscription_manager import injection as inj
from rhsm.config import RhsmConfigParser
from rhsmlib.services import config

from subscription_manager import repolib
from subscription_manager import repofile
from subscription_manager.entcertlib import CONTENT_ACCESS_CERT_TYPE


class ConfigFromString(config.Config):
    def __init__(self, config_string):
        parser = RhsmConfigParserFromString(config_string)
        super(ConfigFromString, self).__init__(parser)


class RhsmConfigParserFromString(RhsmConfigParser):
    def __init__(self, config_string):
        SafeConfigParser.__init__(self)
        self.stringio = six.StringIO(config_string)
        self.readfp(self.stringio)


class TestRepoActionInvoker(fixture.SubManFixture):
    def _stub_content(self, include_content_access=False):
        stub_prod = StubProduct('stub_product',
                                provided_tags="stub-product")

        stub_content = StubContent("a_test_repo",
                                   required_tags="stub-product")

        stub_content2 = StubContent("test_repo_2", required_tags="stub-product")

        stub_ent_cert = StubEntitlementCertificate(stub_prod,
                                                   content=[stub_content])
        stub_prod_cert = StubProductCertificate(stub_prod)

        certs = [stub_ent_cert]
        if include_content_access:
            self.stub_content_access_cert = StubEntitlementCertificate(stub_prod, content=[stub_content, stub_content2],
                                                                       entitlement_type=CONTENT_ACCESS_CERT_TYPE)
            # content access cert is first and last, so naively wrong implementations will prioritize it.
            certs = [self.stub_content_access_cert, stub_ent_cert, self.stub_content_access_cert]
        stub_ent_dir = StubEntitlementDirectory(certs)
        stub_prod_dir = StubProductDirectory([stub_prod_cert])

        inj.provide(inj.ENT_DIR, stub_ent_dir)
        inj.provide(inj.PROD_DIR, stub_prod_dir)

    def test_is_managed(self):
        self._stub_content()
        repo_action_invoker = RepoActionInvoker()
        repo_label = 'a_test_repo'

        im_result = repo_action_invoker.is_managed(repo_label)

        self.assertTrue(im_result)

    def test_get_repo_file(self):
        repo_action_invoker = RepoActionInvoker()

        repo_file = repo_action_invoker.get_repo_file()
        self.assertFalse(repo_file is None)

    def test_get_repos_empty_dirs(self):
        repo_action_invoker = RepoActionInvoker()
        repos = repo_action_invoker.get_repos()
        if repos:
            self.fail("get_repos() should have returned an empty set but did not.")

    def test_get_repos(self):
        self._stub_content(include_content_access=True)
        repo_action_invoker = RepoActionInvoker()
        repos = repo_action_invoker.get_repos()
        self.assertEqual(2, len(repos), 'Should produce two repos')
        matching_repos = [repo for repo in repos if repo.id == 'a_test_repo']
        self.assertEqual(1, len(matching_repos), 'Should only produce one repo for "a_test_repo"')
        repo = matching_repos[0]
        certpath = repo.get('sslclientcert')
        self.assertNotEqual(certpath, self.stub_content_access_cert.path)


PROXY_NO_PROTOCOL = """
[server]
proxy_hostname = fake.server.com
proxy_port = 3129
"""

PROXY_EMPTY_PROTOCOL = """
[server]
proxy_hostname = fake.server.com
proxy_port = 3129
proxy_scheme =
"""

PROXY_HTTP_PROTOCOL = """
[server]
proxy_hostname = fake.server.com
proxy_scheme = http
proxy_port = 3129
"""

PROXY_HTTPS_PROTOCOL = """
[server]
proxy_hostname = fake.server.com
proxy_scheme = https
proxy_port = 3129
"""

PROXY_EXTRA_SCHEME = """
[server]
proxy_hostname = fake.server.com
proxy_scheme = https://
proxy_port = 3129
"""


class RepoTests(unittest.TestCase):
    """
    Tests for the repolib Repo class
    """

    def test_valid_label_for_id(self):
        repo_id = 'valid-label'
        repo = Repo(repo_id)
        self.assertEqual(repo_id, repo.id)

    def test_valid_unicode_just_ascii_label_for_id(self):
        repo_id = u'valid-label'
        repo = Repo(repo_id)
        self.assertEqual(repo_id, repo.id)

    def test_invalid_unicode_label_for_id(self):
        repo_id = u'valid-不明-label'
        repo = Repo(repo_id)
        expected = 'valid----label'
        self.assertEqual(expected, repo.id)

    def test_invalid_label_with_spaces(self):
        repo_id = 'label with spaces'
        repo = Repo(repo_id)
        self.assertEqual('label-with-spaces', repo.id)

    def test_existing_order_is_preserved(self):
        config = (('key 1', 'value 1'), ('key b', 'value b'), ('key 3', 'value 3'))
        repo = Repo('testrepo', config)
        self.assertEqual(config, tuple(repo.items())[:3])

    def test_empty_strings_not_set_in_file(self):
        r = Repo('testrepo', (('proxy', ""),))
        r['proxy'] = ""
        self.assertFalse(("proxy", "") in list(r.items()))

    def test_unknown_property_is_preserved(self):
        existing_repo = Repo('testrepo')
        existing_repo['fake_prop'] = 'fake'
        self.assertTrue(('fake_prop', 'fake') in list(existing_repo.items()))

    @patch.object(repofile, 'conf', ConfigFromString(config_string=PROXY_NO_PROTOCOL))
    def test_http_by_default(self):
        repo = Repo('testrepo')
        r = Repo._set_proxy_info(repo)
        self.assertEqual(r['proxy'], "http://fake.server.com:3129")

    @patch.object(repofile, 'conf', ConfigFromString(config_string=PROXY_EMPTY_PROTOCOL))
    def test_http_by_empty(self):
        repo = Repo('testrepo')
        r = Repo._set_proxy_info(repo)
        self.assertEqual(r['proxy'], "http://fake.server.com:3129")

    @patch.object(repofile, 'conf', ConfigFromString(config_string=PROXY_HTTP_PROTOCOL))
    def test_http(self):
        repo = Repo('testrepo')
        r = Repo._set_proxy_info(repo)
        self.assertEqual(r['proxy'], "http://fake.server.com:3129")

    @patch.object(repofile, 'conf', ConfigFromString(config_string=PROXY_HTTPS_PROTOCOL))
    def test_https(self):
        repo = Repo('testrepo')
        r = Repo._set_proxy_info(repo)
        self.assertEqual(r['proxy'], "https://fake.server.com:3129")

    @patch.object(repofile, 'conf', ConfigFromString(config_string=PROXY_EXTRA_SCHEME))
    def test_extra_chars_in_scheme(self):
        repo = Repo('testrepo')
        r = Repo._set_proxy_info(repo)
        self.assertEqual(r['proxy'], "https://fake.server.com:3129")


class RepoActionReportTests(fixture.SubManFixture):
    def test(self):
        report = repolib.RepoActionReport()
        repo = self._repo(u'a-unicode-content-label', u'A unicode repo name')
        report.repo_updates.append(repo)
        report.repo_added.append(repo)
        deleted_section = u'einige-repo-name'
        deleted_section_2 = u'一些回購名稱'
        report.repo_deleted = [deleted_section, deleted_section_2]

        str(report)

        with fixture.locale_context('de_DE.utf8'):
            str(report)

    def _repo(self, id, name):
        repo = repolib.Repo(repo_id=id)
        repo['name'] = name
        return repo

    def test_empty(self):
        report = repolib.RepoActionReport()
        self.assertEqual(report.updates(), 0)
        '%s' % report
        str(report)

    def test_format(self):
        report = repolib.RepoActionReport()
        repo = self._repo('a-repo-label', 'A Repo Name')
        report.repo_updates.append(repo)
        res = str(report)
        # needs tests run in eng locale, coupled to report format since
        # I managed to typo them
        report_label_regexes = ['^Repo updates$', '^Total repo updates:',
                                '^Updated$', '^Added \(new\)$', '^Deleted$']
        for report_label_regex in report_label_regexes:
            if not re.search(report_label_regex, res, re.MULTILINE):
                self.fail("Expected to match the report label regex  %s but did not" % report_label_regex)


class RepoUpdateActionTests(fixture.SubManFixture):

    def setUp(self):
        super(RepoUpdateActionTests, self).setUp()
        stub_prod = StubProduct("fauxprod", provided_tags="TAG1,TAG2")
        stub_prod2 = StubProduct("fauxprovidedprod", provided_tags="TAG4")
        stub_prod_cert = StubProductCertificate(stub_prod, provided_products=[stub_prod2])
        stub_prod2 = StubProduct("fauxprod2", provided_tags="TAG5,TAG6")
        stub_prod2_cert = StubProductCertificate(stub_prod2)
        prod_dir = StubProductDirectory([stub_prod_cert, stub_prod2_cert])
        inj.provide(inj.PROD_DIR, prod_dir)

        stub_content = [
                StubContent("c1", required_tags="", gpg=None),   # no required tags
                StubContent("c2", required_tags="TAG1", gpg=""),
                StubContent("c3", required_tags="TAG1,TAG2,TAG3"),  # should get skipped
                StubContent("c4", required_tags="TAG1,TAG2,TAG4,TAG5,TAG6",
                    gpg="/gpg.key", url="/$some/$path"),
                StubContent("c5", content_type="file", required_tags="", gpg=None),
                StubContent("c6", content_type="file", required_tags="", gpg=None),
        ]
        self.stub_ent_cert = StubEntitlementCertificate(stub_prod, content=stub_content)
        ent_dir = StubEntitlementDirectory([self.stub_ent_cert])
        inj.provide(inj.ENT_DIR, ent_dir)

        repolib.ConsumerIdentity = StubConsumerIdentity

    def _find_content(self, content_list, name):
        """
        Scan list of content for one with name.
        """
        for content in content_list:
            if content['name'] == name:
                return content
        return None

    def test_override_cache_update_skipped_when_overrides_not_supported_on_server(self):
        inj.provide(inj.OVERRIDE_STATUS_CACHE, Mock())
        override_cache_mock = inj.require(inj.OVERRIDE_STATUS_CACHE)

        mock_uep = Mock()
        mock_uep.supports_resource = Mock(return_value=False)
        mock_uep.get_supported_resources = Mock(return_value=[])
        mock_uep.getCertificates = Mock(return_value=[])
        mock_uep.getCertificateSerials = Mock(return_value=[])
        mock_uep.getRelease = Mock(return_value={'releaseVer': "dummyrelease"})

        self.set_consumer_auth_cp(mock_uep)

        RepoUpdateActionCommand()

        # No cache calls should be made if overrides are not supported
        self.assertFalse(override_cache_mock.read_cache.called)
        self.assertFalse(override_cache_mock.load_status.called)

    def test_overrides_trump_ent_cert(self):
        update_action = RepoUpdateActionCommand()
        update_action.overrides = {'x': {'gpgcheck': 'blah'}}
        r = Repo('x', [('gpgcheck', 'original'), ('gpgkey', 'some_key')])
        self.assertEqual('original', r['gpgcheck'])
        update_action._set_override_info(r)
        self.assertEqual('blah', r['gpgcheck'])
        self.assertEqual('some_key', r['gpgkey'])

    def test_overrides_trump_existing(self):
        update_action = RepoUpdateActionCommand()
        update_action.overrides = {'x': {'gpgcheck': 'blah'}}
        values = [('gpgcheck', 'original'), ('gpgkey', 'some_key')]
        old_repo = Repo('x', values)
        new_repo = Repo(old_repo.id, values)
        update_action._set_override_info(new_repo)
        self.assertEqual('original', old_repo['gpgcheck'])
        update_action.update_repo(old_repo, new_repo)
        self.assertEqual('blah', old_repo['gpgcheck'])
        self.assertEqual('some_key', old_repo['gpgkey'])

    @patch("subscription_manager.repolib.get_repo_file_classes")
    def test_update_when_new_repo(self, mock_get_repo_file_classes):
        mock_file = MagicMock()
        mock_file.CONTENT_TYPES = [None]
        mock_file.fix_content = lambda x: x
        mock_file.section.return_value = None
        mock_class = MagicMock(return_value=mock_file)
        mock_get_repo_file_classes.return_value = [(mock_class, mock_class)]

        def stub_content():
            return [Repo('x', [('gpgcheck', 'original'), ('gpgkey', 'some_key')])]

        update_action = RepoUpdateActionCommand()
        update_action.get_unique_content = stub_content
        update_report = update_action.perform()
        written_repo = mock_file.add.call_args[0][0]
        self.assertEqual('original', written_repo['gpgcheck'])
        self.assertEqual('some_key', written_repo['gpgkey'])
        self.assertEqual(1, update_report.updates())

    @patch("subscription_manager.repolib.get_repo_file_classes")
    def test_update_when_repo_not_modified_on_mutable(self, mock_get_repo_file_classes):
        self._inject_mock_invalid_consumer()
        modified_repo = Repo('x', [('gpgcheck', 'original'), ('gpgkey', 'some_key')])
        server_repo = Repo('x', [('gpgcheck', 'original')])
        mock_file = MagicMock()
        mock_file.CONTENT_TYPES = [None]
        mock_file.fix_content = lambda x: x
        mock_file.section.side_effect = [modified_repo, server_repo]
        mock_class = MagicMock(return_value=mock_file)
        mock_get_repo_file_classes.return_value = [(mock_class, mock_class)]

        def stub_content():
            return [Repo('x', [('gpgcheck', 'new'), ('gpgkey', 'new_key'), ('name', 'test')])]

        update_action = RepoUpdateActionCommand()
        update_action.get_unique_content = stub_content
        current = update_action.perform()
        # confirming that the assessed value does change when repo file
        # is the same as the server value file.
        self.assertEqual('new', current.repo_updates[0]['gpgcheck'])

        # this is the ending server value file.
        written_repo = mock_file.update.call_args[0][0]
        self.assertEqual('new', written_repo['gpgcheck'])
        self.assertEqual(None, written_repo['gpgkey'])

    @patch("subscription_manager.repolib.get_repo_file_classes")
    def test_update_when_repo_modified_on_mutable(self, mock_get_repo_file_classes):
        self._inject_mock_invalid_consumer()
        modified_repo = Repo('x', [('gpgcheck', 'unoriginal'), ('gpgkey', 'some_key')])
        server_repo = Repo('x', [('gpgcheck', 'original')])
        mock_file = MagicMock()
        mock_file.CONTENT_TYPES = [None]
        mock_file.fix_content = lambda x: x
        mock_file.section.side_effect = [modified_repo, server_repo]
        mock_class = MagicMock(return_value=mock_file)
        mock_get_repo_file_classes.return_value = [(mock_class, mock_class)]

        def stub_content():
            return [Repo('x', [('gpgcheck', 'new'), ('gpgkey', 'new_key'), ('name', 'test')])]

        update_action = RepoUpdateActionCommand()
        update_action.get_unique_content = stub_content
        current = update_action.perform()
        # confirming that the assessed value does not change when repo file
        # is different from the server value file.
        self.assertEqual('unoriginal', current.repo_updates[0]['gpgcheck'])

        # this is the ending server value file
        written_repo = mock_file.update.call_args[0][0]
        self.assertEqual('new', written_repo['gpgcheck'])
        self.assertEqual(None, written_repo['gpgkey'])

    def test_no_gpg_key(self):

        update_action = RepoUpdateActionCommand()
        content = update_action.get_all_content(baseurl="http://example.com",
                                                ca_cert=None)
        c1 = self._find_content(content, 'c1')
        self.assertEqual('', c1['gpgkey'])
        self.assertEqual('0', c1['gpgcheck'])

        c2 = self._find_content(content, 'c2')
        self.assertEqual('', c2['gpgkey'])
        self.assertEqual('0', c2['gpgcheck'])

    def test_gpg_key(self):

        update_action = RepoUpdateActionCommand()
        content = update_action.get_all_content(baseurl="http://example.com",
                                                ca_cert=None)
        c4 = self._find_content(content, 'c4')
        self.assertEqual('http://example.com/gpg.key', c4['gpgkey'])
        self.assertEqual('1', c4['gpgcheck'])

    def test_ui_repoid_vars(self):
        update_action = RepoUpdateActionCommand()
        content = update_action.get_all_content(baseurl="http://example.com",
                                            ca_cert=None)
        c4 = self._find_content(content, 'c4')
        self.assertEqual('some path', c4['ui_repoid_vars'])
        c2 = self._find_content(content, 'c2')
        self.assertEqual(None, c2['ui_repoid_vars'])

    def test_tags_found(self):
        update_action = RepoUpdateActionCommand()
        content = update_action.get_unique_content()
        self.assertEqual(3, len(content))

    def test_only_allow_content_of_type_yum(self):
        update_action = RepoUpdateActionCommand()
        content = update_action.get_all_content(baseurl="http://example.com",
                                                ca_cert=None)
        self.assertTrue(self._find_content(content, "c1") is not None)
        self.assertTrue(self._find_content(content, "c5") is None)
        self.assertTrue(self._find_content(content, "c6") is None)

    def test_mutable_property(self):
        update_action = RepoUpdateActionCommand()
        self._inject_mock_invalid_consumer()
        existing_repo = Repo('testrepo')
        existing_repo['metadata_expire'] = 1000
        incoming_repo = {'metadata_expire': 2000}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual(1000, existing_repo['metadata_expire'])

    def test_mutable_property_is_server(self):
        update_action = RepoUpdateActionCommand()
        self._inject_mock_invalid_consumer()
        existing_repo = Repo('testrepo')
        server_val_repo = Repo('servertestrepo')
        existing_repo['metadata_expire'] = 1000
        server_val_repo['metadata_expire'] = 1000
        incoming_repo = {'metadata_expire': 2000}
        update_action.update_repo(existing_repo, incoming_repo, server_val_repo)
        self.assertEqual(2000, existing_repo['metadata_expire'])

    def test_gpgcheck_is_mutable(self):
        update_action = RepoUpdateActionCommand()
        self._inject_mock_invalid_consumer()
        existing_repo = Repo('testrepo')
        existing_repo['gpgcheck'] = "0"
        incoming_repo = {'gpgcheck': "1"}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual("0", existing_repo['gpgcheck'])

    def test_mutable_property_in_repo_but_not_in_cert(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['metadata_expire'] = 1000
        incoming_repo = {}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual(1000, existing_repo['metadata_expire'])

    def test_immutable_property(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['name'] = "meow"
        incoming_repo = {'name': "woof"}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual("woof", existing_repo['name'])

    # If the user removed a mutable property completely, or the property
    # is new in a new version of the entitlement certificate, the new value
    # should get written out.
    def test_unset_mutable_property(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        incoming_repo = {'metadata_expire': 2000}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual(2000, existing_repo['metadata_expire'])

    def test_unset_immutable_property(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        incoming_repo = {'name': "woof"}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertEqual("woof", existing_repo['name'])

    # Test repo on disk has an immutable property set which has since been
    # unset in the new repo definition. This property should be removed.
    def test_set_immutable_property_now_empty(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['proxy_username'] = "blah"
        incoming_repo = {}
        update_action.update_repo(existing_repo, incoming_repo)
        self.assertFalse("proxy_username" in list(existing_repo.keys()))

    def test_set_mutable_property_now_empty_value(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['metadata_expire'] = "blah"
        incoming_repo = {'metadata_expire': ''}
        update_action.update_repo(existing_repo, incoming_repo)
        # re comments in repolib
        # Mutable properties should be added if not currently defined,
        # otherwise left alone.
        self.assertTrue("metadata_expire" in list(existing_repo.keys()))

    def test_set_immutable_property_now_empty_value(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['proxy_username'] = "blah"
        incoming_repo = {'proxy_username': ''}
        update_action.update_repo(existing_repo, incoming_repo)
        # Immutable properties should be always be added/updated,
        # and removed if undefined in the new repo definition.
        self.assertFalse("proxy_username" in list(existing_repo.keys()))

    def test_set_mutable_property_now_none(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['metadata_expire'] = "blah"
        incoming_repo = {'metadata_expire': None}
        update_action.update_repo(existing_repo, incoming_repo)
        # re comments in repolib
        # Mutable properties should be added if not currently defined,
        # otherwise left alone.
        self.assertTrue("metadata_expire" in list(existing_repo.keys()))

    def test_set_mutable_property_now_not_in_cert(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['metadata_expire'] = "blah"
        incoming_repo = {}
        update_action.update_repo(existing_repo, incoming_repo)
        # re comments in repolib
        # Mutable properties should be added if not currently defined,
        # otherwise left alone.
        self.assertTrue("metadata_expire" in list(existing_repo.keys()))

    def test_set_immutable_property_now_none(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['proxy_username'] = "blah"
        incoming_repo = {'proxy_username': None}
        update_action.update_repo(existing_repo, incoming_repo)
        # Immutable properties should be always be added/updated,
        # and removed if undefined in the new repo definition.
        self.assertFalse("proxy_username" in list(existing_repo.keys()))

    def test_set_immutable_property_now_not_in_cert(self):
        self._inject_mock_invalid_consumer()
        update_action = RepoUpdateActionCommand()
        existing_repo = Repo('testrepo')
        existing_repo['proxy_username'] = "blah"
        incoming_repo = {}
        update_action.update_repo(existing_repo, incoming_repo)
        # Immutable properties should be always be added/updated,
        # and removed if undefined in the new repo definition.
        self.assertFalse("proxy_username" in list(existing_repo.keys()))

    def test_overrides_removed_revert_to_default(self):
        update_action = RepoUpdateActionCommand()
        update_action.written_overrides.overrides = {'x': {'gpgcheck': 'blah'}}
        update_action.overrides = {}
        old_repo = Repo('x', [('gpgcheck', 'blah'), ('gpgkey', 'some_key')])
        new_repo = Repo(old_repo.id, [('gpgcheck', 'original'), ('gpgkey', 'some_key')])
        update_action._set_override_info(new_repo)
        # The value from the current repo file (with the old override) should exist pre-update
        self.assertEqual('blah', old_repo['gpgcheck'])
        update_action.update_repo(old_repo, new_repo)
        # Because the override has been removed, the value is reset to the default
        self.assertEqual('original', old_repo['gpgcheck'])
        self.assertEqual('some_key', old_repo['gpgkey'])

    def test_overrides_removed_and_edited(self):
        update_action = RepoUpdateActionCommand()
        update_action.written_overrides.overrides = {'x': {'gpgcheck': 'override_value'}}
        update_action.overrides = {}
        old_repo = Repo('x', [('gpgcheck', 'hand_edit'), ('gpgkey', 'some_key')])
        new_repo = Repo(old_repo.id, [('gpgcheck', 'original'), ('gpgkey', 'some_key')])
        update_action._set_override_info(new_repo)
        # The value from the current repo file (with the old hand edit) should exist pre-update
        self.assertEqual('hand_edit', old_repo['gpgcheck'])
        update_action.update_repo(old_repo, new_repo)
        # Because the current value doesn't match the override, we don't modify it
        self.assertEqual('hand_edit', old_repo['gpgcheck'])
        self.assertEqual('some_key', old_repo['gpgkey'])

    def test_non_default_overrides_added_to_existing(self):
        '''
        Test that overrides for values that aren't found in Repo.PROPERTIES are written
        to existing repos
        '''
        update_action = RepoUpdateActionCommand()
        update_action.written_overrides.overrides = {}
        update_action.overrides = {'x': {'somekey': 'someval'}}
        old_repo = Repo('x', [])
        new_repo = Repo(old_repo.id, [])
        update_action._set_override_info(new_repo)
        update_action.update_repo(old_repo, new_repo)
        self.assertEqual('someval', old_repo['somekey'])

    def test_non_default_override_removed_deleted(self):
        '''
        Test that overrides for values that aren't found in Repo.PROPERTIES are
        removed from redhat.repo once the override is removed
        '''
        update_action = RepoUpdateActionCommand()
        update_action.written_overrides.overrides = {'x': {'somekey': 'someval'}}
        update_action.overrides = {}
        old_repo = Repo('x', [('somekey', 'someval')])
        new_repo = Repo(old_repo.id, [])
        update_action._set_override_info(new_repo)
        update_action.update_repo(old_repo, new_repo)
        self.assertFalse('somekey' in old_repo)


class TidyWriterTests(unittest.TestCase):

    def test_just_newlines_compressed_to_one(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("\n\n\n\n")
        tidy_writer.close()

        self.assertEqual("\n", output.getvalue())

    def test_newline_added_to_eof(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("a line\n")
        tidy_writer.write("another line")
        tidy_writer.close()

        self.assertEqual("a line\nanother line\n", output.getvalue())

    def test_newline_preserved_on_eof(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("a line\n")
        tidy_writer.write("another line\n")
        tidy_writer.close()

        self.assertEqual("a line\nanother line\n", output.getvalue())

    def test_compression_preserves_a_single_blank_line(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("test stuff\n\ntest\n")
        tidy_writer.close()

        self.assertEqual("test stuff\n\ntest\n", output.getvalue())

    def test_newlines_compressed_in_single_write(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("test stuff\n\n\ntest\n")
        tidy_writer.close()

        self.assertEqual("test stuff\n\ntest\n", output.getvalue())

    def test_newlines_compressed_across_writes(self):
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("test stuff\n\n")
        tidy_writer.write("\ntest\n")
        tidy_writer.close()

        self.assertEqual("test stuff\n\ntest\n", output.getvalue())

        # now try the other split
        output = six.StringIO()
        tidy_writer = TidyWriter(output)

        tidy_writer.write("test stuff\n")
        tidy_writer.write("\n\ntest\n")
        tidy_writer.close()

        self.assertEqual("test stuff\n\ntest\n", output.getvalue())


class YumReleaseverSourceTest(fixture.SubManFixture):
    def test_init(self):
        #inj.provide(inj.RELEASE_STATUS_CACHE, Mock())
        #override_cache_mock = inj.require(inj.OVERRIDE_STATUS_CACHE)

        release_source = YumReleaseverSource()
        self.assertEqual(release_source._expansion, None)
        self.assertEqual(release_source.marker, "$releasever")
        self.assertEqual(release_source.marker, release_source.default)

    def test_default(self):
        release_source = YumReleaseverSource()

        exp = release_source.get_expansion()
        self.assertEqual(exp, "$releasever")

    def test_mem_cache_works(self):
        inj.provide(inj.RELEASE_STATUS_CACHE, Mock())
        release_mock = inj.require(inj.RELEASE_STATUS_CACHE)

        release = "MockServer"
        mock_release = {'releaseVer': release}
        release_mock.read_status = Mock(return_value=mock_release)
        release_source = YumReleaseverSource()

        exp = release_source.get_expansion()
        self.assertEqual(exp, release)
        self.assertEqual(release_source._expansion, release)

        exp = release_source.get_expansion()
        self.assertEqual(exp, release)

    def test_mem_cache_pre_cached(self):
        inj.provide(inj.RELEASE_STATUS_CACHE, Mock())
        release_mock = inj.require(inj.RELEASE_STATUS_CACHE)

        release = "MockServer"
        mock_release = {'releaseVer': release}
        release_mock.read_status = Mock(return_value=mock_release)
        release_source = YumReleaseverSource()

        cached_release = "CachedMockServer"
        release_source._expansion = cached_release
        exp = release_source.get_expansion()
        self.assertEqual(exp, cached_release)
        self.assertEqual(release_source._expansion, cached_release)

    def test_read_status_not_set(self):
        inj.provide(inj.RELEASE_STATUS_CACHE, Mock())
        release_mock = inj.require(inj.RELEASE_STATUS_CACHE)

        release = ""
        mock_release = {'releaseVer': release}
        release_mock.read_status = Mock(return_value=mock_release)
        release_source = YumReleaseverSource()

        exp = release_source.get_expansion()

        # we were unset, should return the default
        self.assertEqual(exp, YumReleaseverSource.default)
        # and cache it
        self.assertEqual(release_source._expansion, YumReleaseverSource.default)


class YumReleaseverSourceIsNotEmptyTest(fixture.SubManFixture):
    def test_empty_string(self):
        self.assertFalse(YumReleaseverSource.is_not_empty(""))

    def test_none(self):
        self.assertFalse(YumReleaseverSource.is_not_empty(None))

    def test_empty_list(self):
        self.assertFalse(YumReleaseverSource.is_not_empty([]))

    def test_number(self):
        self.assertTrue(YumReleaseverSource.is_not_empty("7"))

    def test_string(self):
        self.assertTrue(YumReleaseverSource.is_not_empty("Super"))


class YumReleaseverSourceIsSetTest(fixture.SubManFixture):
    def test_none(self):
        self.assertFalse(YumReleaseverSource.is_set(None))

    def test_key_error(self):
        self.assertFalse(YumReleaseverSource.is_set({'not_release_ver': 'blippy'}))

    def test_not_a_dict(self):
        self.assertFalse(YumReleaseverSource.is_set(['some string']))

    def test_releasever_zero(self):
        self.assertFalse(YumReleaseverSource.is_set({'releaseVer': 0}))

    def test_releasever_none(self):
        self.assertFalse(YumReleaseverSource.is_set({'releaseVer': None}))

    def test_releasever_7server(self):
        self.assertTrue(YumReleaseverSource.is_set({'releaseVer': '7Server'}))

    def test_releasever_the_string_none(self):
        # you laugh, but someone thinks that would be an awesome product name.
        self.assertTrue(YumReleaseverSource.is_set({'releaseVer': 'None'}))


class YumRepoFileTest(unittest.TestCase):

    @patch("subscription_manager.repofile.YumRepoFile.create")
    @patch("subscription_manager.repofile.TidyWriter")
    def test_configparsers_equal(self, tidy_writer, stub_create):
        rf = YumRepoFile()
        other = RawConfigParser()
        for parser in [rf, other]:
            parser.add_section('test')
            parser.set('test', 'key', 'val')
        self.assertTrue(rf._configparsers_equal(other))

    @patch("subscription_manager.repofile.YumRepoFile.create")
    @patch("subscription_manager.repofile.TidyWriter")
    def test_configparsers_diff_sections(self, tidy_writer, stub_create):
        rf = YumRepoFile()
        rf.add_section('new_section')
        other = RawConfigParser()
        for parser in [rf, other]:
            parser.add_section('test')
            parser.set('test', 'key', 'val')
        self.assertFalse(rf._configparsers_equal(other))

    @patch("subscription_manager.repofile.YumRepoFile.create")
    @patch("subscription_manager.repofile.TidyWriter")
    def test_configparsers_diff_item_val(self, tidy_writer, stub_create):
        rf = YumRepoFile()
        other = RawConfigParser()
        for parser in [rf, other]:
            parser.add_section('test')
            parser.set('test', 'key', 'val')
        rf.set('test', 'key', 'val2')
        self.assertFalse(rf._configparsers_equal(other))

    @patch("subscription_manager.repofile.YumRepoFile.create")
    @patch("subscription_manager.repofile.TidyWriter")
    def test_configparsers_diff_items(self, tidy_writer, stub_create):
        rf = YumRepoFile()
        other = RawConfigParser()
        for parser in [rf, other]:
            parser.add_section('test')
            parser.set('test', 'key', 'val')
        rf.set('test', 'somekey', 'val')
        self.assertFalse(rf._configparsers_equal(other))

    @patch("subscription_manager.repofile.YumRepoFile.create")
    @patch("subscription_manager.repofile.TidyWriter")
    def test_configparsers_equal_int(self, tidy_writer, stub_create):
        rf = YumRepoFile()
        other = RawConfigParser()
        for parser in [rf, other]:
            parser.add_section('test')
            parser.set('test', 'key', 'val')
        rf.set('test', 'k', 1)
        other.set('test', 'k', '1')
        self.assertTrue(rf._configparsers_equal(other))


# config file is root only, so just fill in a stringbuffer
unset_manage_repos_cfg_buf = """
[server]
hostname = server.example.conf
prefix = /candlepin
[rhsm]
manage_repos =

[rhsmcertd]
certCheckInterval = 240
"""

unset_config = """[server]
hostname = server.example.conf
"""

manage_repos_zero_config = """[rhsm]
manage_repos = 0
"""

manage_repos_bool_config = """[rhsm]
manage_repos = false
"""

manage_repos_not_an_int = """[rhsm]
manage_repos = thisisanint
"""

manage_repos_int_37 = """[rhsm]
manage_repos = 37
"""


class TestManageReposEnabled(fixture.SubManFixture):
    @patch.object(repofile, 'conf', ConfigFromString(config_string=unset_config))
    def test(self):
        # default stub config, no manage_repo defined, uses default
        manage_repos_enabled = repofile.manage_repos_enabled()
        self.assertEqual(manage_repos_enabled, True)

    @patch.object(repofile, 'conf', ConfigFromString(config_string=unset_manage_repos_cfg_buf))
    def test_empty_manage_repos(self):
        manage_repos_enabled = repofile.manage_repos_enabled()
        self.assertEqual(manage_repos_enabled, True)

    @patch.object(repofile, 'conf', ConfigFromString(config_string=manage_repos_zero_config))
    def test_empty_manage_repos_zero(self):
        manage_repos_enabled = repofile.manage_repos_enabled()
        self.assertEqual(manage_repos_enabled, False)

    @patch.object(repofile, 'config', ConfigFromString(config_string=manage_repos_bool_config))
    def test_empty_manage_repos_bool(self):
        manage_repos_enabled = repofile.manage_repos_enabled()
        # Should fail, and return default of 1
        self.assertEqual(manage_repos_enabled, True)

    @patch.object(repofile, 'config', ConfigFromString(config_string=manage_repos_not_an_int))
    def test_empty_manage_repos_not_an_int(self):
        manage_repos_enabled = repofile.manage_repos_enabled()
        # Should fail, and return default of 1
        self.assertEqual(manage_repos_enabled, True)

    @patch.object(repofile, 'conf', ConfigFromString(config_string=manage_repos_int_37))
    def test_empty_manage_repos_int_37(self):
        manage_repos_enabled = repofile.manage_repos_enabled()
        # Should fail, and return default of 1
        self.assertEqual(manage_repos_enabled, True)


AUTO_ENABLE_PKG_PLUGINS_ENABLED = """
[rhsm]
auto_enable_yum_plugins = 1
"""

AUTO_ENABLE_PKG_PLUGINS_DISABLED = """
[rhsm]
auto_enable_yum_plugins = 0
"""

PKG_PLUGIN_CONF_FILE_ENABLED_INT = """
[main]
enabled = 1
"""

PKG_PLUGIN_CONF_FILE_ENABLED_BOOL = """
[main]
enabled = true
"""

PKG_PLUGIN_CONF_FILE_DISABLED_INT = """
[main]
enabled = 0
"""

PKG_PLUGIN_CONF_FILE_DISABLED_BOOL = """
[main]
enabled = false
"""

PKG_PLUGIN_CONF_FILE_WRONG_VALUE = """
[main]
enabled = 4
"""

PKG_PLUGIN_CONF_FILE_INVALID_VALUE = """
[main]
enabled = 1.0
"""

PKG_PLUGIN_CONF_MISSING_MAIN_SECTION = """
[test]
option = value
"""

PKG_PLUGIN_CONF_MISSING_ENABLED_OPTION = """
[main]
option = value
"""

PKG_PLUGIN_CONF_FILE_CORRUPTED = """
[main
enable =
"""


class TestYumPluginManager(unittest.TestCase):
    """
    This class is intended for testing YumPluginManager
    """

    ORIGINAL_DNF_PLUGIN_DIR = YumPluginManager.DNF_PLUGIN_DIR
    ORIGINAL_YUM_PLUGIN_DIR = YumPluginManager.YUM_PLUGIN_DIR
    ORIGINAL_PLUGINS = YumPluginManager.PLUGINS

    def init_yum_plugin_conf_files(self, conf_string):
        """
        Mock configuration files of plugins
        """
        yum_tmp_dir = tempfile.mkdtemp()
        f, plug_file_name_01 = tempfile.mkstemp(prefix='', suffix='.conf', dir=yum_tmp_dir, text=True)
        f, plug_file_name_02 = tempfile.mkstemp(prefix='', suffix='.conf', dir=yum_tmp_dir, text=True)
        with open(plug_file_name_01, "w") as plug_file_01:
            plug_file_01.write(conf_string)
        with open(plug_file_name_02, "w") as plug_file_02:
            plug_file_02.write(conf_string)
        self.plug_file_name_01 = plug_file_name_01
        self.plug_file_name_02 = plug_file_name_02
        self.tmp_dir = yum_tmp_dir
        YumPluginManager.YUM_PLUGIN_DIR = yum_tmp_dir
        YumPluginManager.PLUGINS = [
            plug_file_name_01.replace(yum_tmp_dir, '').replace('.conf', ''),
            plug_file_name_02.replace(yum_tmp_dir, '').replace('.conf', '')
        ]

    def restore_yum_plugin_conf_files(self):
        """
        Restore original constants in YumPluginManager
        """
        YumPluginManager.YUM_PLUGIN_DIR = self.ORIGINAL_YUM_PLUGIN_DIR
        YumPluginManager.PLUGINS = self.ORIGINAL_PLUGINS
        os.unlink(self.plug_file_name_01)
        os.unlink(self.plug_file_name_02)
        os.rmdir(self.tmp_dir)

    def init_dnf_plugin_conf_files(self, conf_string):
        """
        Mock configuration files of plugins
        """
        dnf_tmp_dir = tempfile.mkdtemp()
        f, plug_file_name_01 = tempfile.mkstemp(prefix='', suffix='.conf', dir=dnf_tmp_dir, text=True)
        f, plug_file_name_02 = tempfile.mkstemp(prefix='', suffix='.conf', dir=dnf_tmp_dir, text=True)
        with open(plug_file_name_01, "w") as plug_file_01:
            plug_file_01.write(conf_string)
        with open(plug_file_name_02, "w") as plug_file_02:
            plug_file_02.write(conf_string)
        self.plug_file_name_01 = plug_file_name_01
        self.plug_file_name_02 = plug_file_name_02
        self.tmp_dir = dnf_tmp_dir
        YumPluginManager.DNF_PLUGIN_DIR = dnf_tmp_dir
        YumPluginManager.PLUGINS = [
            plug_file_name_01.replace(dnf_tmp_dir, '').replace('.conf', ''),
            plug_file_name_02.replace(dnf_tmp_dir, '').replace('.conf', '')
        ]

    def restore_dnf_plugin_conf_files(self):
        """
        Restore original constants in YumPluginManager
        """
        YumPluginManager.DNF_PLUGIN_DIR = self.ORIGINAL_DNF_PLUGIN_DIR
        YumPluginManager.PLUGINS = self.ORIGINAL_PLUGINS
        os.unlink(self.plug_file_name_01)
        os.unlink(self.plug_file_name_02)
        os.rmdir(self.tmp_dir)

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_disabled_yum_plugin(self):
        """
        Test automatic enabling of configuration files of disabled plugins
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_DISABLED_INT)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = yum_plugin_config.getint('main', 'enabled')
            self.assertEqual(is_plugin_enabled, 1)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_DISABLED))
    def test_not_enabling_disabled_yum_plugin(self):
        """
        Test not enabling (disabled in rhsm.conf) of configuration files of disabled plugins
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_DISABLED_BOOL)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 0)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = yum_plugin_config.getboolean('main', 'enabled')
            self.assertEqual(is_plugin_enabled, False)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_disabled_dnf_plugin(self):
        """
        Test automatic enabling of configuration files of disabled plugins
        """
        self.init_dnf_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_DISABLED_INT)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            dnf_plugin_config = ConfigParser()
            result = dnf_plugin_config.read(YumPluginManager.DNF_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = dnf_plugin_config.getint('main', 'enabled')
            self.assertEqual(is_plugin_enabled, 1)
        self.restore_dnf_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_enabled_yum_plugin_int(self):
        """
        Test automatic enabling of configuration files of already enabled plugins
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_ENABLED_INT)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 0)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = yum_plugin_config.getint('main', 'enabled')
            self.assertEqual(is_plugin_enabled, 1)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_enabled_yum_plugin_bool(self):
        """
        Test automatic enabling of configuration files of already enabled plugins
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_ENABLED_BOOL)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 0)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            # The file was not modified. We have to read value with with getboolean()
            is_plugin_enabled = yum_plugin_config.getboolean('main', 'enabled')
            self.assertEqual(is_plugin_enabled, True)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_yum_plugin_with_invalid_values(self):
        """
        Test automatic enabling of configuration files of already enabled plugins
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_INVALID_VALUE)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = yum_plugin_config.getint('main', 'enabled')
            self.assertEqual(is_plugin_enabled, 1)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_yum_plugin_with_wrong_conf(self):
        """
        Test automatic enabling of configuration files of already plugins with wrong values in conf file.
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_WRONG_VALUE)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)
        for plugin_conf_file_name in YumPluginManager.PLUGINS:
            yum_plugin_config = ConfigParser()
            result = yum_plugin_config.read(YumPluginManager.YUM_PLUGIN_DIR + '/' + plugin_conf_file_name + '.conf')
            self.assertGreater(len(result), 0)
            is_plugin_enabled = yum_plugin_config.getint('main', 'enabled')
            self.assertEqual(is_plugin_enabled, 1)
        self.restore_yum_plugin_conf_files()

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_yum_plugin_with_corrupted_conf_file(self):
        """
        This test only test YumPluginManager.enable_yum_plugins() can survive reading of corrupted
        yum plugin configuration file
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_FILE_CORRUPTED)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 0)

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_yum_plugin_with_missing_main_section(self):
        """
        This test only test YumPluginManager.enable_yum_plugins() can survive reading of corrupted
        yum plugin configuration file (missing main section)
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_MISSING_MAIN_SECTION)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)

    @patch.object(repolib, 'conf', ConfigFromString(config_string=AUTO_ENABLE_PKG_PLUGINS_ENABLED))
    def test_enabling_yum_plugin_with_missing_enabled_option(self):
        """
        This test only test YumPluginManager.enable_yum_plugins() can survive reading of corrupted
        yum plugin configuration file (missing option 'enabled')
        """
        self.init_yum_plugin_conf_files(conf_string=PKG_PLUGIN_CONF_MISSING_ENABLED_OPTION)
        plugin_list = YumPluginManager.enable_pkg_plugins()
        self.assertEqual(len(plugin_list), 2)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# Authors: James Bowes <jbowes@redhat.com>
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# hack to allow bytes/strings to be interpolated w/ unicode values (gettext gives us bytes)
# Without this, for example, "Формат: %s\n" % u"foobar" will fail with UnicodeDecodeError
# See http://stackoverflow.com/a/29832646/6124862 for more details
import six
import sys
if six.PY2:
    reload(sys)
    sys.setdefaultencoding('utf-8')

enable_debug = False


def excepthook_base(exc_type, exc_value, exc_traceback):
    # something failed before we even got logging setup
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return

    if enable_debug:
        sys.__excepthook__(exc_type, exc_value, exc_traceback)

    # something fundamental failed... how quiet should we be?
    sys.exit(0)


sys.excepthook = excepthook_base

import syslog
import dbus
import dbus.service
import dbus.glib
import decorator
import logging
import traceback

from subscription_manager import ga_loader
ga_loader.init_ga()

# from gi.repository import GObject
log = logging.getLogger("rhsm-app.rhsmd")

from subscription_manager import logutil
logutil.init_logger()


# If we get here, we should be okay to use define excepthook that
# uses our logging. Set this up before we do injection init, since
# that has a lot of potential failures.
def excepthook_logging(exc_type, exc_value, exc_traceback):
    framelist = traceback.format_exception(exc_type, exc_value, exc_traceback)
    log.error("Unhandled rhsmd exception caught by the logging excepthook: %s",
              "".join(framelist))

    return excepthook_base(exc_type, exc_value, exc_traceback)


sys.excepthook = excepthook_logging

from subscription_manager.ga import GObject as ga_GObject
from subscription_manager.injectioninit import init_dep_injection
init_dep_injection()

from subscription_manager.branding import get_branding
from subscription_manager.injection import require, IDENTITY, CERT_SORTER, RHSM_ICON_CACHE
from rhsmlib.facts.hwprobe import ClassicCheck
from subscription_manager.i18n_optparse import OptionParser, \
    WrappedIndentedHelpFormatter, USAGE
from subscription_manager.cert_sorter import RHSM_VALID, \
        RHSM_EXPIRED, RHSM_WARNING, RHSM_PARTIALLY_VALID, \
        RHN_CLASSIC, RHSM_REGISTRATION_REQUIRED
from subscription_manager.utils import print_error, is_process_running

from rhsm.config import initConfig
from rhsmlib.services import config

conf = config.Config(initConfig())


def debug(msg):
    """
    Print debug messages to console or rhsm.log
    :param msg:
    :return:
    """
    if enable_debug:
        log.debug(msg)
        print(msg)
    else:
        log.debug(msg)


def in_warning_period(sorter):
    """
    Is any entitlement certificate expired?
    :param sorter:
    :return:
    """
    for entitlement in sorter.valid_entitlement_certs:
        if entitlement.is_expiring():
            return True
    return False


def pre_check_status(force_signal):
    """
    Pre check status of the system. Is it registered against classic. Is it registered?
    :param force_signal: When this is not None, then this value is returned
    :return: Specified force_signal or RHN_CLASSIC or RHSM_REGISTRATION_REQUIRED or None
    """
    if force_signal is not None:
        debug("forcing status signal from cli arg")
        return force_signal

    if ClassicCheck().is_registered_with_classic():
        debug("System is already registered to another entitlement system")
        return RHN_CLASSIC

    identity = require(IDENTITY)
    sorter = require(CERT_SORTER)

    if not identity.is_valid() and not sorter.has_entitlements():
        debug("The system is not currently registered.")
        return RHSM_REGISTRATION_REQUIRED
    return None


def check_status(force_signal):
    """
    When pre_check_status does not return anything, then status is retrieved from candlepin server
    using ComplianceManager
    :param force_signal: When force_signal is not None, then this signal will be returned
    :return: Some signal
    """
    pre_result = pre_check_status(force_signal)
    if pre_result is not None:
        return pre_result

    sorter = require(CERT_SORTER)

    return sorter.get_status_for_icon()


def check_if_ran_once(checker, loop):
    """
    Callback function used for checking if the d-bus has been called at least once
    :param checker: instance of StatusChecker
    :param loop: instance GObject.Mainloop
    :return:
    """
    if checker.has_run:
        msg = "D-Bus com.redhat.SubscriptionManager.EntitlementStatus.check_status called once, exiting"
        debug(msg)
        loop.quit()
    return True


def timeout_cb(loop, checker):
    """
    Timeout callback method is called only after timeout defined in StatusChecker
    :param loop: instance of GObject.MainLoop
    :param checker: instance of StatusChecker
    :return: None
    """
    debug("Timeout reached. Checking reason to continue ...")
    if checker.keep_alive:
        debug("Not terminating rhsmd, because CLI option --keep-alive was set")
        return

    if is_rhsm_icon_running():
        debug("Not terminating rhsmd, because rhsm-icon is running")
        return

    debug("No reason found. Terminating")
    loop.quit()


@decorator.decorator
def ensure_exit(func, *args, **kwargs):
    """
    A decorator ensuring the decorated method exits the main loop after
    running (even if it fails with an exception)
    :param func: the method that will be decorated. Should be a method of
    StatusChecker
    :return:
    """
    try:
        return func(*args, **kwargs)
    finally:
        try:
            status_checker = args[0]  # Should be StatusChecker instance
            status_checker.has_run = True
            status_checker.watchdog()
        except IndexError:
            log.exception("Cannot get reference to StatusChecker "
                          "instance.\nUnable to quit out of main loop.\n "
                          "rhsmd may not quit automatically.")
        except AttributeError:
            log.exception("Failed to quit mainloop.\nWas 'ensure_exit' used "
                          "to decorate a method of a class that does not have"
                          "a main loop?")


def is_rhsm_icon_running():
    """
    This function checks if rhsm-icon process is running.
    :return: It returns True, when rhsm-icon is running. Otherwise return False.
    """
    debug('Checking if rhsm-icon process is running')
    ret = is_process_running('rhsm-icon')
    if ret:
        debug('Process rhsm-icon is running')
    else:
        debug('Process rhsm-icon is not running')
    return ret


class StatusChecker(dbus.service.Object):
    # NOTE: All methods of this class that need to exit the main loop
    # will need the annotation @ensure_exit. To avoid issues with the
    # dbus.service.method annotation, add annotations earlier in the
    # annotation stack.

    # Timeout in seconds (same default value as is used for rhsmd.processTimeout)
    TIMEOUT = 300

    def __init__(self, bus, keep_alive, force_signal, loop):
        name = dbus.service.BusName("com.redhat.SubscriptionManager", bus)
        dbus.service.Object.__init__(self, name, "/EntitlementStatus")
        self.has_run = False
        # this will get set after first invocation
        self.rhsm_icon_cache = require(RHSM_ICON_CACHE)
        self.keep_alive = keep_alive
        self.force_signal = force_signal
        self.loop = loop
        # Read process timeout from configuration file
        self.read_rhsm_config()
        # the callback function will try to terminate main loop after timout is reached
        ga_GObject.timeout_add_seconds(self.TIMEOUT, timeout_cb, loop, self)

    def read_rhsm_config(self):
        """
        Try to read specific option: rhsmd.processTimeout from configuration file rhm.conf.
        :return: None
        """
        debug('Trying to get processTimeout option from rhsm.conf')
        if 'rhsmd' in conf and 'processTimeout' in conf['rhsmd']:
            timeout = conf['rhsmd']['processTimeout']
            try:
                self.TIMEOUT = int(timeout)
            except ValueError:
                debug('Unable to convert "%s" into number; using default value: %d' % (timeout, self.TIMEOUT))
            else:
                debug('New timeout is set according rhsmd.processTimeout: %s' % self.TIMEOUT)
        else:
            debug('Unable to read rhsmd.processTimeout from rhsm.conf; using default value: %d' % self.TIMEOUT)

    @dbus.service.signal(
        dbus_interface='com.redhat.SubscriptionManager.EntitlementStatus',
        signature='i')
    def entitlement_status_changed(self, status_code):
        log.debug("D-Bus signal com.redhat.SubscriptionManager.EntitlementStatus.entitlement_status_changed emitted")
        debug("signal fired! code is " + str(status_code))

    def watchdog(self):
        """
        This is so we can guarantee exit after the D-Bus stuff is done, since
        certain parts of that are async
        :return: None
        """
        if not self.keep_alive:
            ga_GObject.idle_add(check_if_ran_once, self, self.loop)

    @dbus.service.method(
        dbus_interface="com.redhat.SubscriptionManager.EntitlementStatus",
        out_signature='i')
    @ensure_exit
    def check_status(self):
        """
        returns: 0 if entitlements are valid, 1 if not valid,
                 2 if close to expiry
        """
        log.debug("D-Bus interface com.redhat.SubscriptionManager.EntitlementStatus.check_status called")
        status = check_status(self.force_signal)
        cached_status = self.rhsm_icon_cache.read_cache_only()
        if cached_status and status != cached_status:
            debug("Validity status changed, fire signal in check_status")
            self.entitlement_status_changed(status)
        self.rhsm_icon_cache.data = status
        self.rhsm_icon_cache.write_cache()
        return status

    @dbus.service.method(
            dbus_interface="com.redhat.SubscriptionManager.EntitlementStatus",
            in_signature='i')
    @ensure_exit
    def update_status(self, status):
        log.debug(
            "D-Bus interface com.redhat.SubscriptionManager.EntitlementStatus.update_status called with status = %s"
            % status
        )
        pre_result = pre_check_status(self.force_signal)
        if pre_result is not None:
            status = pre_result
        # At comment time, update status is called every time we start the GUI. So we use
        # a persistent cache to ensure we fire a signal only when the status changes.
        cached_status = self.rhsm_icon_cache.read_cache_only()
        if cached_status and status != cached_status:
            debug("Validity status changed, fire signal")
            self.entitlement_status_changed(status)
        self.rhsm_icon_cache.data = status
        self.rhsm_icon_cache.write_cache()


def parse_force_signal(cli_arg):
    """
    Try to validate provided signal on CLI
    :param cli_arg: string with signal
    :return: Code of signal or None
    """
    if cli_arg is None:
        return None

    cli_arg = cli_arg.lower().strip()

    if cli_arg == "valid":
        return RHSM_VALID
    elif cli_arg == "expired":
        return RHSM_EXPIRED
    elif cli_arg == "warning":
        return RHSM_WARNING
    elif cli_arg == "partial":
        return RHSM_PARTIALLY_VALID
    elif cli_arg == "classic":
        return RHN_CLASSIC
    elif cli_arg == "registration_required":
        return RHSM_REGISTRATION_REQUIRED
    else:
        print_error("Invalid force option: %s" % cli_arg)
        sys.exit(-1)


def log_syslog(level, msg):
    """
    Log message to syslog
    :param level: Level of debug print
    :param msg: String with message
    :return: None
    """
    syslog.openlog("rhsmd")
    syslog.syslog(level, msg)
    log.debug("rhsmd: %s" % msg)
    if enable_debug:
        print(msg)


def main():
    """
    Main method of rhsmd providing D-Bus API com.redhat.SubscriptionManager
    :return: None
    """
    debug("rhsmd started")
    log.info("D-Bus API: com.redhat.SubscriptionManager provided by rhsmd is deprecated")
    log.info("Consider using D-Bus API: com.redhat.RHSM1 provided by rhsm.service")

    parser = OptionParser(usage=USAGE,
                          formatter=WrappedIndentedHelpFormatter())
    parser.add_option("-d", "--debug", dest="debug",
            help="Display debug messages", action="store_true", default=False)
    parser.add_option("-k", "--keep-alive", dest="keep_alive",
            help="Stay running (don't shut down after the first dbus call)",
            action="store_true", default=False)
    parser.add_option("-s", "--syslog", dest="syslog",
            help="Run standalone and log result to syslog",
            action="store_true", default=False)
    parser.add_option("-f", "--force-signal", dest="force_signal",
            help="Force firing of a signal " +
            "(valid, expired, warning, partial, classic or registration_required)")
    parser.add_option("-i", "--immediate", dest="immediate",
            action="store_true", default=False,
            help="Fire forced signal immediately (requires --force-signal)")

    options, args = parser.parse_args()

    force_signal = parse_force_signal(options.force_signal)

    if options.immediate and force_signal is None:
        print_error("--immediate must be used with --force-signal")
        sys.exit(-2)

    global enable_debug
    enable_debug = options.debug

    # short-circuit dbus initialization
    if options.syslog:
        log.debug("logging subscription status to syslog")
        status = check_status(force_signal)
        if status == RHSM_EXPIRED:
            log_syslog(syslog.LOG_NOTICE,
                       "This system is missing one or more subscriptions. " +
                       "Please run subscription-manager for more information.")
        elif status == RHSM_PARTIALLY_VALID:
            log_syslog(syslog.LOG_NOTICE,
                       "This system is missing one or more subscriptions " +
                       "to fully cover its products. " +
                       "Please run subscription-manager for more information.")
        elif status == RHSM_WARNING:
            log_syslog(syslog.LOG_NOTICE,
                       "This system's subscriptions are about to expire. " +
                       "Please run subscription-manager for more information.")
        elif status == RHN_CLASSIC:
            log_syslog(syslog.LOG_INFO,
                       get_branding().RHSMD_REGISTERED_TO_OTHER)
        elif status == RHSM_REGISTRATION_REQUIRED:
            log_syslog(syslog.LOG_NOTICE,
                       "In order for Subscription Manager to provide your " +
                       "system with updates, your system must be registered " +
                       "with the Customer Portal. Please enter your Red Hat " +
                       "login to ensure your system is up-to-date.")

        # Return an exit code for the program. having valid entitlements is
        # good, so it gets an exit status of 0.
        return status

    # we are not running from cron here, so unset the excepthook
    # though, we may be running from cli, or as a dbus activation. For
    # cli, we should traceback. For dbus, we should try to log it and
    # raise dbus exception?
    sys.excepthook = sys.__excepthook__

    system_bus = dbus.SystemBus()
    loop = ga_GObject.MainLoop()
    checker = StatusChecker(system_bus, options.keep_alive, force_signal, loop)

    if options.immediate:
        checker.entitlement_status_changed(force_signal)

    loop.run()


if __name__ == "__main__":
    main()



from __future__ import print_function, division, absolute_import

# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

"""
This module contains implementation of D-Bus object representing system purpose.
It uses interface: com.redhat.RHSM1.Syspurpose and path:
/com/redhat/RHSM1/Syspurpose
"""

import dbus
import json
import logging

from rhsmlib.dbus import constants, base_object, util, dbus_utils
from rhsmlib.services import syspurpose
from syspurpose.files import SyspurposeStore
from subscription_manager import injection as inj

from subscription_manager.injectioninit import init_dep_injection
from subscription_manager.i18n import Locale

init_dep_injection()

log = logging.getLogger(__name__)


class SyspurposeDBusObject(base_object.BaseObject):
    """
    A D-Bus object interacting with subscription-manager to get
    information about current system purpose.
    """
    default_dbus_path = constants.SYSPURPOSE_DBUS_PATH
    interface_name = constants.SYSPURPOSE_INTERFACE

    def __init__(self, conn=None, object_path=None, bus_name=None):
        super(SyspurposeDBusObject, self).__init__(conn=conn, object_path=object_path, bus_name=bus_name)

    @util.dbus_service_method(
        constants.SYSPURPOSE_INTERFACE,
        in_signature='s',
        out_signature='s')
    @util.dbus_handle_exceptions
    def GetSyspurpose(self, locale, sender=None):
        """
        D-Bus method for getting current system purpose
        :param locale: string with locale
        :param sender:
        :return: json representation of system purpose contents
        """
        syspurpose_path = '/etc/rhsm/syspurpose/syspurpose.json'

        locale = dbus_utils.dbus_to_python(locale, expected_type=str)
        Locale.set(locale)

        syspurpose_store = SyspurposeStore.read(syspurpose_path)

        try:
            contents = syspurpose_store.contents
        except Exception as err:
            raise dbus.DBusException(str(err))

        return json.dumps(contents)

    @util.dbus_service_method(
        constants.SYSPURPOSE_INTERFACE,
        in_signature='',
        out_signature='s'
    )
    @util.dbus_handle_exceptions
    def GetSyspurposeStatus(self, sender=None):
        cp = inj.require(inj.CP_PROVIDER).get_consumer_auth_cp()
        systempurpose = syspurpose.Syspurpose(cp)
        syspurpose_status = systempurpose.get_syspurpose_status()['status']
        return systempurpose.get_overall_status(syspurpose_status)

    @util.dbus_service_signal(
        constants.SYSPURPOSE_INTERFACE,
        signature=''
    )
    @util.dbus_handle_exceptions
    def SyspurposeChanged(self):
        """
        Signal fired, when system purpose is created/deleted/changed
        :param sender:
        :return: None
        """
        log.debug("D-Bus signal %s emitted" % constants.SYSPURPOSE_INTERFACE)
        return None



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import mock
import six

from subscription_manager import plugins
from subscription_manager import base_plugin


# SubManPlugin
# plugin_key

# BasePluginManager
#  create
#  load test plugins
#  load no plugins
#  load plugins from bad plugin dir
#  load plugins no conduits
# _import_plugins dupe modules
# _load_plugin_module no modules
# _load_plugin_module import errors
# _load_plugin_module no api version
# _load_plugin_module unmet api ver dep
# add_plugins_from_module same plugin name
# add_plugins_from_module empty module
# add_plugins_from_module no classes
# add_plugins_from_module no SubManPlugin classes
# add_plugins_from_module classes imported into name space
#                         classes with bad new()
#                         no config for class
#                         bad config for class
#                         no enabled in config for class
#                         disabled in config for class
#                         enabled set on and off
#                         plugin classes that set a conf themselves
#                         verify all classes end up in found_plugin_classes
#                         enabled plugins added to _plugins
#                         disabled classes end up tracked as disabled
#                         verify we fail module if any plugin class in that module fails
#                         verify we dont fail module if plugin is disabled
# add_plugin_class
#                plugin_key is set properly
#                plugins instaniated
#                right exceptions on plugin init fail
#                loaded plugins get added to _plugins
#                dupe classes raise PluginException
#                valid plugins added to _plugin_classes
#                verify we handle no slots
#                handle _hook name variantions (ie, foo_hook ok, foo_hook_blip is not
#                   verify we handle '' slot names
#                verify we append found hook to slot_to_funcs
#                verify we DTRT on non callable instances that match
#                        aka (self.post_install_hook = 'asdasd')
#                        we should ignore non callables
#                        verify hooks are bound or class/static methods?
#                if we find a hook, verify clazz.found_slots_for_hooks is True
#                   make sure we handle multiple hooks (if any hooks are used,
#                   we mark class as used
#               can we verify we set the class.used and not the class itself?
#               after adding classes, verify our method of finding used classes is correct
# run
#      check how slot_name in kwargs works
#      make sure we work with no kwargs, None kwargs, empty kwargs
#      verify we dont return any value (can you assert on that? thing = this_does_not_return()
#      check we log  method call correctly, with correct arg
#      verify SlotNameException
#      handle/warn/assert no slots
#      handle slot name not being in slots
#      check that we handle no hooks for a slot
#      verify we handle multiple hooks for a slot
#      handle exeptions on hook invocation, and run rest of hooks
#       handle exceptions on conduit init (probably dont need to
#       conduit look up could fail if we munge self._slot_to_conduit
#     test variables Exceptions on conduit init
#      verify we dont reload plugin config on each conduit init
#      verify conduit logger gets created with correct class name
#        - how would we verufy conduits get there data attributes set after
#          super()?
# Conduits
#     verify we ignore unused kwargs
#
# reporting stuff
#     get_plugins returns all found plugins, including disabled ones
#     get_plugins accounts for used plugins
#
# PluginManager
#   init with no/default args get's our config
#   init with specified search_path/conf_path use those
#   verify _get_conduits only returns class objects, not instances
#   verify _get_conduits is not empty
#
# get_plugin_manager
#   multiple invocations of get_plugin_manager returnt he same PluginManager object
#   can we test that get_plugin_manager is not imported into a local namespace?


# this test class heavily uses mock to simulate the "default" case
# through PluginManager. The main issue being that PluginConfigs
# init'ed based on the PluginClass passed in, and by defaults, looks
# for a real config file somewhere.
class TestBasePluginManagerAddPluginsFromModule(unittest.TestCase):

    def setUp(self):
        self.base_manager = plugins.BasePluginManager()
        self.assertTrue(isinstance(self.base_manager, plugins.BasePluginManager))

    def test_empty_mock_module(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'
        self.base_manager.add_plugins_from_module(mock_module)
        # that was not a module, nor a plugin module, nor had any plugin classes
        # so we better not have any plugins found
        self.assertEqual({}, self.base_manager._slot_to_funcs)
        self.assertEqual({}, self.base_manager._plugin_classes)

    def test_no_classes(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        self.base_manager.add_plugins_from_module(mock_module)
        # that was not a module, nor a plugin module, nor had any plugin classes
        # so we better not have any plugins found
        self.assertEqual({}, self.base_manager._slot_to_funcs)
        self.assertEqual({}, self.base_manager._plugin_classes)

    def test_unrelated_classes(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        class NotAPluginClass(object):
            pass

        mock_module.NotAPluginClass = NotAPluginClass
        self.base_manager.add_plugins_from_module(mock_module)
        self.assertEqual({}, self.base_manager._slot_to_funcs)
        self.assertEqual({}, self.base_manager._plugin_classes)

    def test_plugin_class_no_conf(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

        mock_module.PluginClass = PluginClass

        # there is no config for this plugin, so it should raise a
        # PluginConfigException
        self.assertRaises(plugins.PluginConfigException,
                          self.base_manager.add_plugins_from_module,
                          mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # these shouldn't get populated for this cases
        self.assertEqual({}, self.base_manager._slot_to_funcs)
        self.assertEqual({}, self.base_manager._plugin_classes)

    def test_plugin_config_fail(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

        mock_module.PluginClass = PluginClass
        # this config is empty, so we will fail to read its config
#        plugin_conf = PluginConfig(PluginClass.get_plugin_key())

        self.assertRaises(plugins.PluginConfigException,
                          self.base_manager.add_plugins_from_module,
                           mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)
        # these shouldn't get populated for this cases
        self.assertEqual({}, self.base_manager._slot_to_funcs)
        self.assertEqual({}, self.base_manager._plugin_classes)

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_plugin_config_disabled(self, mock_plugin_conf):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = False

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

        # unused in this case
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()

        mock_module.PluginClass = PluginClass
        # class wont be added, but no exceptions expected.
        # no plugin will be loaded though
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)
        # these shouldn't get populated for this cases
        self.assertEqual({}, self.base_manager._slot_to_funcs)

        # we shouldn't be in _plugins, since this was disabled
        self.assertFalse(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # however, we were found
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_plugin_config_enabled(self, mock_plugin_conf):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we have to mock this guy, since we dont really want to provide
        # a direct api for providing an alternate config, since that controls
        # if we enable the plugin or not. But maybe that's unneeded and we
        # could just pass add_plugins_from_module a dict of PluginConfs...
        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = True

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

        # we validate the plugins_conf plugin key matches...
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()
        mock_module.PluginClass = PluginClass

        # should be able to load this guy
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # these shouldn't get populated for this cases, mp slots
        self.assertEqual({}, self.base_manager._slot_to_funcs)

        # we should be in _plugins, since this plugin was enabled
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # and we account the plugin class
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

        # we don't provide any hooks (and we haven't setup any slots), so this
        # plugin is "unused"
        self.assertFalse(PluginClass.found_slots_for_hooks)

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_plugin_unmatch_hooks(self, mock_plugin_conf):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we have to mock this guy, since we dont really want to provide
        # a direct api for providing an alternate config, since that controls
        # if we enable the plugin or not. But maybe that's unneeded and we
        # could just pass add_plugins_from_module a dict of PluginConfs...
        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = True

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

            def this_doesnt_exist_hook(self):
                pass

        # we validate the plugins_conf plugin key matches...
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()
        mock_module.PluginClass = PluginClass

        # should be able to load this guy
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # these shouldn't get populated for this cases, mp slots
        self.assertEqual({}, self.base_manager._slot_to_funcs)

        # we should be in _plugins, since this plugin was enabled
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # and we account the plugin class
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

        # we don't provide any hooks (and we haven't setup any slots), so this
        # plugin is "unused"
        self.assertFalse(PluginClass.found_slots_for_hooks)

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_plugin_hooks_with_conduits(self, mock_plugin_conf):
        self.base_manager.conduits = [plugins.FactsConduit]
        # to refill these
        self.base_manager._populate_slots()

        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we have to mock this guy, since we dont really want to provide
        # a direct api for providing an alternate config, since that controls
        # if we enable the plugin or not. But maybe that's unneeded and we
        # could just pass add_plugins_from_module a dict of PluginConfs...
        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = True

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

            def this_doesnt_exist_hook(self):
                pass

            def post_facts_collection_hook(self):
                pass

        # we validate the plugins_conf plugin key matches...
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()
        mock_module.PluginClass = PluginClass

        # should be able to load this guy
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # we provide conduits with slots, so we should have known slots now
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_funcs)
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_conduit)
        self.assertEqual(plugins.FactsConduit,
                          self.base_manager._slot_to_conduit['post_facts_collection'])

        # we should be in _plugins, since this plugin was enabled
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # and we account the plugin class
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

        # we found hooks that map to slots
        self.assertTrue(PluginClass.found_slots_for_hooks)

        funcs = self.base_manager._slot_to_funcs['post_facts_collection']
        # we find our hook mapped to this slot
        self.assertTrue('post_facts_collection_hook' in [x.__name__ for x in funcs])

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_with_conduits_no_matching_hooks(self, mock_plugin_conf):
        self.base_manager.conduits = [plugins.FactsConduit]
        # to refill these
        self.base_manager._populate_slots()

        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we have to mock this guy, since we dont really want to provide
        # a direct api for providing an alternate config, since that controls
        # if we enable the plugin or not. But maybe that's unneeded and we
        # could just pass add_plugins_from_module a dict of PluginConfs...
        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = True

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

            def this_doesnt_exist_hook(self):
                pass

        # we validate the plugins_conf plugin key matches...
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()
        mock_module.PluginClass = PluginClass

        # should be able to load this guy
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # we provide conduits with slots, so we should have known slots now
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_funcs)
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_conduit)
        self.assertEqual(plugins.FactsConduit,
                          self.base_manager._slot_to_conduit['post_facts_collection'])

        # we should be in _plugins, since this plugin was enabled
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # and we account the plugin class
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

        # we found no hooks that map to slots
        self.assertFalse(PluginClass.found_slots_for_hooks)

        funcs = self.base_manager._slot_to_funcs['post_facts_collection']
        # we dont find a hook mapped to this slot
        self.assertFalse('post_facts_collection_hook' in [x.__name__ for x in funcs])

    @mock.patch('subscription_manager.plugins.PluginConfig')
    def test_with_conduits_non_callable_hooks(self, mock_plugin_conf):
        self.base_manager.conduits = [plugins.FactsConduit]
        # to refill these
        self.base_manager._populate_slots()

        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        mock_conf_instance = mock_plugin_conf.return_value
        mock_conf_instance.is_plugin_enabled.return_value = True

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

            # attr with a hook name, that is not callable
            post_facts_collection_hook = True

        # we validate the plugins_conf plugin key matches...
        mock_conf_instance.plugin_key = PluginClass.get_plugin_key()
        mock_module.PluginClass = PluginClass

        # should be able to load this guy
        self.base_manager.add_plugins_from_module(mock_module)

        # we try to load a plugin class for this module, so it should
        # be in the list of tracked modules
        self.assertTrue(mock_module in self.base_manager._modules)

        # we provide conduits with slots, so we should have known slots now
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_funcs)
        self.assertTrue('post_facts_collection' in self.base_manager._slot_to_conduit)
        self.assertEqual(plugins.FactsConduit,
                          self.base_manager._slot_to_conduit['post_facts_collection'])

        # we should be in _plugins, since this plugin was enabled
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugins)

        # and we account the plugin class
        self.assertTrue(PluginClass.get_plugin_key() in self.base_manager._plugin_classes)

        # we found no hooks that map to slots
        self.assertFalse(PluginClass.found_slots_for_hooks)

        funcs = self.base_manager._slot_to_funcs['post_facts_collection']
        # we dont find a hook mapped to this slot
        self.assertFalse('post_facts_collection_hook' in [x.__name__ for x in funcs])


# This is more of a functional test, but for plugins I think that this is okay
# uses test plugins from test/plugins
class TestPluginManager(unittest.TestCase):
    def setUp(self):
        self.module_dir = os.path.join(os.path.dirname(__file__), "plugins")
        self.manager = plugins.PluginManager(self.module_dir, self.module_dir)

    def test_load_plugin_with_no_api_version(self):
        module = os.path.join(self.module_dir, "no_api_version.py")
        self.assertRaises(plugins.PluginModuleImportApiVersionMissingException,
                          self.manager._load_plugin_module_file,
                           module)

    def test_load_plugin_with_old_api_version(self):
        module = os.path.join(self.module_dir, "old_api_version.py")
        self.assertRaises(plugins.PluginModuleImportApiVersionException,
                          self.manager._load_plugin_module_file,
                        module)

    def test_load_plugin(self):
        module = os.path.join(self.module_dir, "dummy_plugin.py")
        plugin_module = self.manager._load_plugin_module_file(module)
        self.assertEqual("dummy_plugin", plugin_module.__name__)

    def test_no_config_plugin(self):
        self.assertRaises(plugins.PluginConfigException,
                          plugins.PluginConfig,
                          "config_plugin.BadConfigPlugin",
                           self.module_dir)

    def test_bad_config_plugin(self):
        self.assertRaises(plugins.PluginConfigException,
                          plugins.PluginConfig,
                          "config_plugin.BadConfigPlugin",
                          self.module_dir)

    def test_good_config_plugin(self):
        parser = plugins.PluginConfig("config_plugin.GoodConfigPlugin",
                                      self.module_dir)
        self.assertTrue(parser)


class TestPluginManagerLoadPluginsFromModule(unittest.TestCase):
    def setUp(self):
        self.module_dir = os.path.join(os.path.dirname(__file__), "plugins")
        self.manager = plugins.PluginManager("some/search/path", "some/config/path")
        self.manager.plugin_conf_path = self.module_dir
        self.manager.search_path = self.module_dir

    def test_load_plugin(self):
        module = os.path.join(self.module_dir, "dummy_plugin.py")
        plugin_module = self.manager._load_plugin_module_file(module)
        self.assertEqual("dummy_plugin", plugin_module.__name__)
        self.manager.add_plugins_from_module(plugin_module)
        self.assertEqual(1, len(self.manager._slot_to_funcs['post_product_id_install']))
        self.assertEqual(0, len(self.manager._slot_to_funcs['pre_product_id_install']))

    def test_load_plugins_with_same_class_name(self):
        module = os.path.join(self.module_dir, "dummy_plugin.py")
        module2 = os.path.join(self.module_dir, "dummy_plugin_2.py")
        plugin_modules = []
        plugin_modules.append(self.manager._load_plugin_module_file(module))
        plugin_modules.append(self.manager._load_plugin_module_file(module2))
        self.manager.add_plugins_from_modules(plugin_modules)
        self.assertEqual(2, len(self.manager._slot_to_funcs['post_product_id_install']))

    def test_load_plugin_from_module_bad_config(self):
        module_file = os.path.join(self.module_dir, "config_plugin.py")
        module = self.manager._load_plugin_module_file(module_file)
        self.assertRaises(plugins.PluginConfigException,
                          self.manager.add_plugins_from_module,
                          module)

    def test_disabled_plugin(self):
        module_file = os.path.join(self.module_dir, "disabled_plugin.py")
        module = self.manager._load_plugin_module_file(module_file)
        self.manager.add_plugins_from_module(module)
        self.assertEqual(0, len(self.manager._plugins))

    def test_run_no_such_slot(self):
        module_file = os.path.join(self.module_dir, "dummy_plugin.py")
        self.manager.search_path = self.module_dir
        self.manager.plugin_conf_path = self.module_dir
        module = self.manager._load_plugin_module_file(module_file)
        self.manager.add_plugins_from_module(module)
        self.assertRaises(plugins.SlotNameException,
                          self.manager.run,
                        'this_is_a_slot_that_doesnt_exist')


# sub class for testing just for easier init
class PluginConfigForTest(plugins.PluginConfig):
    def __init__(self, plugin_key, enabled):
        super(PluginConfigForTest, self).__init__(plugin_key)
        self.parser.add_section("main")
        self.parser.set("main", "enabled", enabled)


class TestPluginManagerConfigMap(unittest.TestCase):
    class PluginClass(base_plugin.SubManPlugin):
        __module__ = "some_module"

    def setUp(self):
        self.manager = plugins.BasePluginManager()
        self.plugin_config = PluginConfigForTest(self.PluginClass.get_plugin_key(),
                                           enabled='1')
        self.plugin_to_config_map = {self.PluginClass.get_plugin_key(): self.plugin_config}

    def test_plugin_no_config_in_map(self):
        # we look in the map, but can't find anything, so we
        # get a PluginConfigException
        broken_map = {'you_wont_find_this': self.plugin_config}
        self.assertRaises(plugins.PluginConfigException,
                          self.manager.add_plugin_class,
                          self.PluginClass,
                          plugin_to_config_map=broken_map)


class TestPluginManagerConfigMapAllHooks(TestPluginManagerConfigMap):
    # example of how a all_hooks class might work
    class PluginClass(base_plugin.SubManPlugin):
        all_hooks = True
        __module__ = "some_module"

        def __getattr__(self, name):
            if name.endswith('_hook'):
                return self.generic_hook_handler
            raise AttributeError

        def generic_hook_handler(self, conduit):
            pass


class TestPluginManagerConfigMapNotCallable(TestPluginManagerConfigMap):
    # example of how a all_hooks class might work
    class PluginClass(base_plugin.SubManPlugin):
        __module__ = "some_module"

        # non callable hook
        some_non_callable_hook = True

    def test_plugin(self):
        self.manager.add_plugin_class(self.PluginClass,
                                      plugin_to_config_map=self.plugin_to_config_map)


class TestPluginManagerReporting(unittest.TestCase):
    class ConduitPluginManager(plugins.BasePluginManager):
        _conduit_list = []

        def _get_conduits(self):
            return self._conduit_list

    def setUp(self):
        #plugin_class_names = ['Plugin1', 'Plugin2', 'Plugin3']
        plugin_class_names = [str(x) for x in range(0, 10)]
        self.plugin_classes = []

        def test_hook(self):
            pass

        conduit_classes = []
        plugin_classes = []
        for plugin_class_name in plugin_class_names:
            plugin_class = type('PluginClass%s' % plugin_class_name,
                                (base_plugin.SubManPlugin,),
                                 {'test_%s_hook' % plugin_class_name: test_hook})
            conduit_class = type('Conduit%s' % plugin_class_name,
                                 (plugins.BaseConduit,),
                                 {'slots': ['test_%s' % plugin_class_name]})
            plugin_classes.append(plugin_class)
            conduit_classes.append(conduit_class)

        for plugin_class in plugin_classes:
            self.plugin_classes.append(plugin_class)

        self.ConduitPluginManager._conduit_list = conduit_classes
        self.manager = self.ConduitPluginManager()

    def test_factory(self):
        plugin_to_config_map = {}
        for plugin_class in self.plugin_classes:
            plugin_config = PluginConfigForTest(plugin_class.get_plugin_key(),
                                                enabled='1')
            plugin_to_config_map[plugin_class.get_plugin_key()] = plugin_config

        for plugin_class in self.plugin_classes:
            self.manager.add_plugin_class(plugin_class,
                                          plugin_to_config_map=plugin_to_config_map)

        self.assertEqual(10, len(self.manager.get_plugins()))
        self.assertEqual(10, len(self.manager.get_slots()))


#functional
class TestPluginManagerRun(unittest.TestCase):
    def setUp(self):
        self.module_dir = os.path.join(os.path.dirname(__file__), "plugins")
        self.manager = plugins.PluginManager("some/search/path", "some/config/path")
        self.manager.conduits = [plugins.ProductConduit]
        self.manager._populate_slots()
        self.manager.plugin_conf_path = self.module_dir
        self.manager.search_path = self.module_dir
        module = os.path.join(self.module_dir, "dummy_plugin.py")
        plugin_module = self.manager._load_plugin_module_file(module)
        self.manager.add_plugins_from_module(plugin_module)

    def test_dummy_run(self):
        self.manager.run("post_product_id_install", product_list=[])

    def test_bad_conduit(self):
        self.assertRaises(TypeError, self.manager.run,
                          "post_product_id_install",
                           not_an_actual_arg=None)

    def test_hook_raises_exception(self):
        class ExceptionalPluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

            def post_product_id_install_hook(self, conduit):
                raise IndexError

        plugin_config = PluginConfigForTest(ExceptionalPluginClass.get_plugin_key(),
                                            enabled='1')

        plugin_to_config_map = {ExceptionalPluginClass.get_plugin_key(): plugin_config}
        self.manager.add_plugin_class(ExceptionalPluginClass,
                                      plugin_to_config_map=plugin_to_config_map)
        self.assertRaises(IndexError, self.manager.run,
                          'post_product_id_install', product_list=[])


class TestPluginManagerRunIter(TestPluginManagerRun):
    def setUp(self):
        super(TestPluginManagerRunIter, self).setUp()

        # add dummy 2 and 3, so we have multiple hooks registered for the same
        # slot
        module = os.path.join(self.module_dir, "dummy_plugin_2.py")
        plugin_module = self.manager._load_plugin_module_file(module)
        self.manager.add_plugins_from_module(plugin_module)

        module = os.path.join(self.module_dir, "dummy_plugin_3.py")
        plugin_module = self.manager._load_plugin_module_file(module)
        self.manager.add_plugins_from_module(plugin_module)
        # add

    def test_dummy_runiter(self):
        for runner in self.manager.runiter("post_product_id_install", product_list=[]):
            runner.run()

    def test_iter_wrapper(self):
        class Wrapper(object):
            def __init__(self, runner):
                self.runner = runner
                self.runner_func = self.runner.func
                self.runner_conduit = self.runner.conduit
                self.status = 0

            def update(self):
                self.runner.run()
                self.status = 1

        for runner in self.manager.runiter("post_product_id_install", product_list=[]):
            wrapper = Wrapper(runner)
            wrapper.update()

    def test_update_content_iter(self):
        reports = set()
        reports.add("Started with this")

        ent_source = []

        for runner in self.manager.runiter("update_content",
                                           reports=reports,
                                           ent_source=ent_source):
            runner.run()


class BaseConduitTest(unittest.TestCase):
    conf_buf = ""

    def setUp(self):
        self.conf_io = six.StringIO(self.conf_buf)
        self.conduit = self._conduit()

    def _conduit(self):
        mock_module = mock.Mock()
        mock_module.__name__ = 'mock_module'

        # we check that the plugin's thinks it is from the same
        # module as the module we pass
        class PluginClass(base_plugin.SubManPlugin):
            __module__ = "mock_module"

        plugin_config = plugins.PluginConfig(PluginClass.get_plugin_key)
        plugin_config.parser.readfp(self.conf_io)

        #
        conduit = plugins.BaseConduit(PluginClass,
                                      plugin_config)
        return conduit


class TestBaseConduitEmptyUsesDefaults(BaseConduitTest):
    def test_default_boolean_false(self):
        val = self.conduit.conf_bool("main", "enabled", False)
        self.assertEqual(False, val)

    def test_default_boolean_true(self):
        val = self.conduit.conf_bool("main", "enabled", True)
        self.assertEqual(True, val)

    def test_bad_default_boolean(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_bool,
                           "main", "enabled", "not a bool")

    def test_boolean_no_section(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_bool,
                          'this_section_is_not_real', 'enabled')

    def test_default_int(self):
        val = self.conduit.conf_int("main", "enabled", 1)
        self.assertEqual(1, val)

    def test_bad_default_int(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_int,
                          "main", "enabled", "not an int")

    def test_default_float(self):
        val = self.conduit.conf_float("main", "enabled", 1.0)
        self.assertEqual(1.0, val)

    def test_bad_default_float(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_float,
                          "main", "enabled", "not a float")

    def test_default_string(self):
        val = self.conduit.conf_string("main", "enabled", "a string")
        self.assertEqual("a string", val)

    def test_string_no_section(self):
        val = self.conduit.conf_string('this_section_is_not_real', 'enabled')
        self.assertTrue(val is None)


class TestBaseConduitDefaultConfig(BaseConduitTest):
    conf_buf = """
[main]
enabled=False
"""


class TestBaseConduitConfig(BaseConduitTest):
    conf_buf = """
[main]
enabled=True
street=place
notabool=0.7

[section1]
string_value=Foo
int_value=37
not_an_int=justastring
not_a_float=sdfadf
float_value=1.0
bool_value=True
"""

    def test_enabled(self):
        val = self.conduit.conf_bool("main", "enabled")
        self.assertTrue(val)

    def test_enabled_default(self):
        val = self.conduit.conf_bool("main", "enabled", False)
        self.assertTrue(val)

    def test_default_boolean_true(self):
        val = self.conduit.conf_bool("main", "enabled", True)
        self.assertTrue(val)

    def test_bad_default_boolean(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_bool,
                           "main", "notabool", "not a bool")

    def test_boolean_no_section(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_bool,
                          'this_section_is_not_real', 'enabled')

    def test_int(self):
        val = self.conduit.conf_int("section1", "int_value")
        self.assertEqual(37, val)

    def test_int_default(self):
        val = self.conduit.conf_int("section1", "int_value", 37)
        self.assertEqual(37, val)

    def test_bad_default_int(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_int,
                          "section1", "not_an_int", "not an int")

    def test_default_float(self):
        val = self.conduit.conf_float("section1", "float_value", 1.0)
        self.assertEqual(1.0, val)

    def test_bad_default_float(self):
        self.assertRaises(ValueError,
                          self.conduit.conf_float,
                          "section1", "not_a_float", "not a float")

    def test_string(self):
        val = self.conduit.conf_string("section1", "string_value")
        self.assertEqual("Foo", val)

    def test_default_string(self):
        val = self.conduit.conf_string("section1", "string_value", "bar")
        self.assertEqual("Foo", val)

    def test_string_no_section(self):
        val = self.conduit.conf_string('this_section_is_not_real', 'enabled')
        self.assertTrue(val is None)


class TestVersionChecks(unittest.TestCase):
    def test_parse_version(self):
        maj_ver, min_ver = plugins.parse_version("1.0")
        self.assertEqual(1, maj_ver)
        self.assertEqual(0, min_ver)

    def test_api_versions_equal(self):
        self.assertTrue(plugins.api_version_ok("1.0", "1.0"))

    def test_api_version_old_minor(self):
        self.assertTrue(plugins.api_version_ok("1.1", "1.0"))

    def test_api_version_old_major(self):
        self.assertFalse(plugins.api_version_ok("1.0", "0.9"))

    def test_api_version_new(self):
        self.assertFalse(plugins.api_version_ok("1.0", "1.1"))


class StubPluginClass(base_plugin.SubManPlugin):
    pass


class TestProductConduit(unittest.TestCase):
    def test_product_conduit(self):
        conduit = plugins.ProductConduit(StubPluginClass, product_list=[])
        self.assertEqual([], conduit.product_list)


class TestProductUpdateConduit(unittest.TestCase):
    def test_product_update_conduit(self):
        conduit = plugins.ProductUpdateConduit(StubPluginClass, product_list=[])
        self.assertEqual([], conduit.product_list)


class TestFactsConduit(unittest.TestCase):
    def test_facts_conduit(self):
        conduit = plugins.FactsConduit(StubPluginClass, facts={})
        self.assertEqual({}, conduit.facts)


class TestUpdateContentConduit(unittest.TestCase):
    def test_content_plugin_conduit(self):
        mock_reports = mock.Mock()

        # out ent source is a empty list (of mock entitlements)
        mock_ent_source = []
        conduit = plugins.UpdateContentConduit(StubPluginClass,
                                               reports=mock_reports,
                                               ent_source=mock_ent_source)
        self.assertEqual(mock_reports, conduit.reports)


class TestRegistrationConduit(unittest.TestCase):
    def test_registration_conduit(self):
        conduit = plugins.RegistrationConduit(StubPluginClass,
                                              name="name",
                                              facts={})
        self.assertEqual("name", conduit.name)
        self.assertEqual({}, conduit.facts)


class TestPostRegistrationConduit(unittest.TestCase):
    def test_post_registration_conduit(self):
        conduit = plugins.PostRegistrationConduit(StubPluginClass,
                                                  consumer={'uuid': 'some_uuid'},
                                                  facts={})
        self.assertEqual("some_uuid", conduit.consumer['uuid'])
        self.assertEqual({}, conduit.facts)


class TestSubscriptionConduit(unittest.TestCase):
    def test_subscription_conduit(self):
        conduit = plugins.SubscriptionConduit(StubPluginClass,
                                              consumer_uuid="123456789",
                                              pool_id="4444",
                                              quantity=4)
        self.assertEqual("123456789", conduit.consumer_uuid)
        self.assertEqual(4, conduit.quantity)
        self.assertEqual("4444", conduit.pool_id)


class TestPostSubscriptionConduit(unittest.TestCase):
    def test_post_subscription_conduit(self):
        conduit = plugins.PostSubscriptionConduit(StubPluginClass,
                                                  consumer_uuid="123456789",
                                                  entitlement_data={})
        self.assertEqual("123456789", conduit.consumer_uuid)
        self.assertEqual({}, conduit.entitlement_data)


class TestAutoAttachConduit(unittest.TestCase):
    def test_auto_attach_conduit(self):
        conduit = plugins.AutoAttachConduit(StubPluginClass, "a-consumer-uuid")
        self.assertEqual("a-consumer-uuid", conduit.consumer_uuid)


class TestPostAutoAttachConduit(unittest.TestCase):
    def test_post_auto_attach_conduit(self):
        conduit = plugins.PostAutoAttachConduit(StubPluginClass,
                                                "a-consumer-uuid",
                                                {})
        self.assertEqual("a-consumer-uuid", conduit.consumer_uuid)
        self.assertEqual({}, conduit.entitlement_data)


class BasePluginException(unittest.TestCase):
    """At least create and raise all the exceptions."""
    e = plugins.PluginException

    def raise_exception(self):
        raise self.e

    def test_exception(self):
        self.assertRaises(self.e,
                          self.raise_exception)


class TestPluginException(BasePluginException):
    def test_add_message(self):
        # the base PluginException is expected to just ignore any args
        # so just tested we dont raise anything in this case
        self.e("hello exception")


class TestPluginImportException(BasePluginException):
    e = plugins.PluginImportException

    def raise_exception(self):
        raise self.e("module_file", "module_name", "import failed")

    def test(self):
        try:
            self.raise_exception()
        except self.e as exp:
            self.assertEqual("module_file", exp.module_file)
            self.assertEqual("module_name", exp.module_name)

    def test_str(self):
        try:
            self.raise_exception()
        except self.e as exp:
            buf = str(exp)
            lines = buf.splitlines()
            # last line is Message:...
            self.assertEqual("Message: import failed", lines[-1])


class TestPluginModuleImportException(TestPluginImportException):
    e = plugins.PluginModuleImportException


class TestPluginModuleImportApiVersionMissingException(TestPluginImportException):
    e = plugins.PluginModuleImportApiVersionMissingException


class TestPluginModuleImportApiVersionException(TestPluginImportException):
    e = plugins.PluginModuleImportApiVersionException

    def raise_exception(self):
        raise self.e("module_file",
                     "module_name",
                     "module_ver",
                     "api_ver",
                     "import failed")


class TestPluginConfigException(BasePluginException):
    e = plugins.PluginConfigException

    def raise_exception(self):
        raise self.e("plugin_name",
                     "msg")

    def test_str(self):
        try:
            self.raise_exception()
        except self.e as exp:
            buf = str(exp)
            lines = buf.splitlines()
            # last line is Message:...
            self.assertEqual("Message: msg", lines[-1])


class TestSlotNameException(BasePluginException):
    e = plugins.SlotNameException

    def raise_exception(self):
        raise self.e("slot_name")

    def test(self):
        try:
            self.raise_exception()
        except self.e as exp:
            self.assertEqual("slot_name", exp.slot_name)
            self.assertTrue("slot_name" in str(exp))



from __future__ import print_function, division, absolute_import

import os

# ../../gui/data/glade/
ourfile = __file__
GTK_BUILDER_FILES_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__),
                                                      "../../gui/data/glade/"))
GTK_BUILDER_FILES_SUFFIX = "glade"

GTK_COMPAT_VERSION = "2"

__all__ = [GTK_BUILDER_FILES_DIR,
           GTK_BUILDER_FILES_SUFFIX]
